#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Sabari Jaganathan (@sajagana) <sajagana@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_l3out_template
short_description: Manage L3Outs on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage L3Outs on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Sabari Jaganathan (@sajagana)
options:
  template:
    description:
    - The name of the L3Out template.
    type: str
    aliases: [ l3out_template ]
    required: true
  name:
    description:
    - The name of the L3Out.
    type: str
  uuid:
    description:
    - The UUID of the L3Out.
    - This parameter is required when the L3Out O(name) needs to be updated.
    type: str
  description:
    description:
    - The description of the L3Out.
    - Providing an empty string will remove the O(description="") from the L3Out.
    type: str
  vrf:
    description:
    - The VRF associated with the L3Out.
    type: dict
    suboptions:
      name:
        description:
        - The name of the VRF.
        required: true
        type: str
      schema:
        description:
        - The name of the schema.
        required: true
        type: str
      template:
        description:
        - The name of the template.
        required: true
        type: str
  l3_domain:
    description:
    - The name of the L3 Domain.
    type: str
  target_dscp:
    description:
    - The DSCP Level of the L3Out.
    type: str
    choices:
      - af11
      - af12
      - af13
      - af21
      - af22
      - af23
      - af31
      - af32
      - af33
      - af41
      - af42
      - af43
      - cs0
      - cs1
      - cs2
      - cs3
      - cs4
      - cs5
      - cs6
      - cs7
      - expedited_forwarding
      - unspecified
      - voice_admit
  pim:
    description:
    - The protocol independent multicast (PIM) flag of the L3Out.
    - By default, PIM is disabled. To enable the PIM, Layer 3 Multicast must be enabled on the O(vrf).
    type: bool
  interleak:
    description:
    - The name of the Route Map Policy for Route Control that needs to be associated with Interleak route map.
    - Providing an empty string will remove the O(interleak="") from the L3Out.
    type: str
  static_route_redistribution:
    description:
    - The name of the Route Map Policy for Route Control that needs to be associated with Static Route Redistribution route map.
    - Providing an empty string will remove the O(static_route_redistribution="") from the L3Out.
    type: str
    aliases: [ static_route ]
  connected_route_redistribution:
    description:
    - The name of the Route Map Policy for Route Control that needs to be associated with Connected Route Redistribution route map.
    - Providing an empty string will remove the O(connected_route_redistribution="") from the L3Out.
    type: str
    aliases: [ connected_route ]
  attached_host_route_redistribution:
    description:
    - The name of the Route Map Policy for Route Control that needs to be associated with Attached Host Route Redistribution route map.
    - Providing an empty string will remove the O(attached_host_route_redistribution="") from the L3Out.
    type: str
    aliases: [ attached_host_route ]
  bgp:
    description:
    - The BGP routing protocol configuration of the L3Out.
    type: dict
    suboptions:
      state:
        description:
        - Use C(enabled) to configure the BGP routing protocol.
        - Use C(disabled) to remove the BGP routing protocol.
        type: str
        default: enabled
        choices: [enabled, disabled]
      inbound_route_map:
        description:
        - The name of the Route Map Policy for Route Control that needs to be associated with inbound route map.
        - Providing an empty string will remove the O(bgp.inbound_route_map="") from the L3Out.
        type: str
        aliases: [ import_route, inbound_route ]
      outbound_route_map:
        description:
        - The name of the Route Map Policy for Route Control that needs to be associated with outbound route map.
        - Providing an empty string will remove the O(bgp.outbound_route_map="") from the L3Out.
        type: str
        aliases: [ export_route, outbound_route ]
      route_dampening_ipv4:
        description:
        - The name of the Route Map Policy for Route Control that needs to be associated with Route Dampening IPv4 route map.
        - Providing an empty string will remove the O(bgp.route_dampening_ipv4="") from the L3Out.
        type: str
        aliases: [ dampening_ipv4 ]
      route_dampening_ipv6:
        description:
        - The name of the Route Map Policy for Route Control that needs to be associated with Route Dampening IPv6 route map.
        - Providing an empty string will remove the O(bgp.route_dampening_ipv6="") from the L3Out.
        type: str
        aliases: [ dampening_ipv6 ]
  ospf:
    description:
    - The OSPF routing protocol configuration of the L3Out.
    type: dict
    suboptions:
      state:
        description:
        - Use C(enabled) to configure the OSPF routing protocol.
        - Use C(disabled) to remove the OSPF routing protocol.
        type: str
        default: enabled
        choices: [enabled, disabled]
      area_id:
        description:
        - The area id of the OSPF area.
        - This option is required when the O(ospf.state=enabled).
        type: str
      area_type:
        description:
        - The area type of the OSPF area.
        - This option is required when the O(ospf.state=enabled).
        type: str
        choices: [regular, stub, nssa]
      cost:
        description:
        - The cost of the OSPF area.
        - Defaults to 1 when unset during creation.
        type: int
      originate_summary_lsa:
        description:
        - This option is for OSPF NSSA (not-so-stubby area) or Stub area.
        - When this option is disabled, not only Type 4 and 5, but also Type 3 LSAs are not sent into the NSSA or Stub area by the border leaf.
        - Instead, the border leaf creates and sends a default route to the area.
        - If there is no Type 3 LSA in this area in the first place, a default route is not created.
        type: bool
        aliases: [ originate_lsa ]
      send_redistributed_lsas:
        description:
        - This option is for the OSPF NSSA (not-so-stubby area).
        - When this option is disabled, the redistributed routes are not sent into this NSSA area from the border leaf.
        - This is typically used when the O(ospf.originate_summary_lsa=false).
        - Because disabling the O(ospf.originate_summary_lsa) option creates and sends a default route to the NSSA or Stub area.
        type: bool
        aliases: [ redistributed_lsas ]
      suppress_forwarding_addr_translated_lsa:
        description:
        - This option is for OSPF NSSA (not-so-stubby area).
        - When an OSPF NSSA ABR (Area Border Router) translates a Type-7 LSA into a Type-5 LSA to send it across non-NSSA areas.
        - It typically includes the IP address of the originator ASBR (Autonomous System Boundary Router) as a forwarding address.
        - However, if an OSPF router receiving the Type-5 LSA lacks a route to this forwarding address.
        - The route may not be installed in the router's route table.
        - Enabling this option prevents the ABR from adding a forwarding address during the Type-7 to Type-5 translation, thereby avoiding this issue.
        type: bool
        aliases: [ suppress_fa_lsa ]
      originate_default_route:
        description:
        - The Originate Default Route option in an L3Out configuration allows the ACI fabric to advertise a default route (0.0.0.0/0) to external networks.
        - Providing an empty string will remove the O(ospf.originate_default_route) from the L3Out.
        type: str
        choices: [ only, in_addition, "" ]
      originate_default_route_always:
        description:
        - Enabling this option will set the O(ospf.originate_default_route=only) when the O(ospf.originate_default_route) unset during creation.
        - This option is applicable only if OSPF is configured on the L3Out.
        type: bool
        aliases: [ always ]
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the L3Out template.
- The O(vrf) must exist before using this module in your playbook.
  Use M(cisco.mso.mso_schema_template_vrf) to create the VRF.
- The O(l3_domain) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_l3_domain) to create the L3Out domain.
seealso:
- module: cisco.mso.mso_schema_template_vrf
- module: cisco.mso.ndo_template
- module: cisco.mso.ndo_l3_domain
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new L3Out object without BGP and OSPF
  cisco.mso.ndo_l3out_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    l3out_template: l3out_template
    name: "l3out_1"
    vrf:
      name: "VRF1"
      schema: "Schema1"
      template: "Template1"
    state: "present"

- name: Query a L3Out object with name
  cisco.mso.ndo_l3out_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    l3out_template: l3out_template
    name: "l3out_1"
    state: "query"
  register: query_l3out_name

- name: Update a L3Out object name with UUID
  cisco.mso.ndo_l3out_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    l3out_template: l3out_template
    uuid: "{{ query_l3out_name.current.uuid }}"
    name: "l3out_1_updated"
    description: "updated description"
    state: "present"

- name: Create a new L3Out object with BGP and OSPF
  cisco.mso.ndo_l3out_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    l3out_template: l3out_template
    name: "l3out_1_new"
    vrf:
      name: "VRF1"
      schema: "Schema1"
      template: "Template1"
    routing_protocols: ["bgp", "ospf"]
    bgp:
      inbound_route_map: "ans_route_map"
      outbound_route_map: "ans_route_map"
      route_dampening_ipv4: "ans_route_map"
      route_dampening_ipv6: "ans_route_map"
    ospf:
      area_id: "0.0.0.1"
      area_type: "regular"
      cost: 1
      send_redistributed_lsas: true
      originate_summary_lsa: true
      suppress_forwarding_addr_translated_lsa: true
      originate_default_route: "only"
      originate_default_route_always: false
    interleak: "ans_route_map"
    static_route_redistribution: "ans_route_map"
    connected_route_redistribution: "ans_route_map"
    attached_host_route_redistribution: "ans_route_map"
    state: "present"

- name: Update a L3Out object BGP and OSPF values
  cisco.mso.ndo_l3out_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    l3out_template: l3out_template
    name: "l3out_1_new"
    vrf:
      name: "VRF1"
      schema: "Schema1"
      template: "Template1"
    routing_protocols: ["bgp", "ospf"]
    bgp:
      route_dampening_ipv4: "ans_route_map_2"
      route_dampening_ipv6: "ans_route_map_2"
    ospf:
      area_id: "0.0.0.2"
      area_type: "stub"
      cost: 3
      send_redistributed_lsas: true
      originate_summary_lsa: true
      suppress_forwarding_addr_translated_lsa: true
    interleak: "ans_route_map"
    static_route_redistribution: "ans_route_map"
    connected_route_redistribution: "ans_route_map"
    attached_host_route_redistribution: "ans_route_map"
    state: "present"

- name: Clear bgp and ospf routing protocol from the existing L3Out
  cisco.mso.ndo_l3out_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    l3out_template: l3out_template
    name: "l3out_1_new"
    vrf:
      name: "VRF1"
      schema: "Schema1"
      template: "Template1"
    routing_protocol: ["bgp"]
    bgp:
      state: disabled
    ospf:
      state: disabled
    state: "present"

- name: Query a L3Out object with uuid
  cisco.mso.ndo_l3out_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    l3out_template: l3out_template
    uuid: "{{ query_l3out_name.current.uuid }}"
    state: "query"
  register: query_l3out_name

- name: Query all L3Out objects
  cisco.mso.ndo_l3out_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    l3out_template: l3out_template
    state: "query"
  register: query_all_l3out

- name: Delete a L3Out object with name
  cisco.mso.ndo_l3out_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    l3out_template: l3out_template
    name: "l3out_1"
    state: "absent"

- name: Delete a L3Out object with uuid
  cisco.mso.ndo_l3out_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    l3out_template: l3out_template
    uuid: "{{ query_l3out_name.current.uuid }}"
    state: "absent"
"""

RETURN = r"""
"""

import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair
from ansible_collections.cisco.mso.plugins.module_utils.constants import TARGET_DSCP_MAP, ORIGINATE_DEFAULT_ROUTE, L3OUT_ROUTING_PROTOCOLS
from ansible_collections.cisco.mso.plugins.module_utils.utils import generate_api_endpoint


def get_routing_protocol(existing_protocol, ospf_state, bgp_state):
    protocols = set()

    if bgp_state == "enabled" or (bgp_state == "ignore" and "bgp" in L3OUT_ROUTING_PROTOCOLS.get(existing_protocol)):
        protocols.add("bgp")

    if ospf_state == "enabled" or (ospf_state == "ignore" and "ospf" in L3OUT_ROUTING_PROTOCOLS.get(existing_protocol)):
        protocols.add("ospf")

    return "".join(protocols) if len(protocols) < 2 else L3OUT_ROUTING_PROTOCOLS.get("".join(protocols))


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", required=True, aliases=["l3out_template"]),
        name=dict(type="str"),
        uuid=dict(type="str"),
        description=dict(type="str"),
        vrf=dict(
            type="dict",
            options=dict(
                name=dict(type="str", required=True),
                schema=dict(type="str", required=True),
                template=dict(type="str", required=True),
            ),
        ),
        l3_domain=dict(type="str"),
        target_dscp=dict(type="str", choices=list(TARGET_DSCP_MAP)),
        pim=dict(type="bool"),
        interleak=dict(type="str"),
        static_route_redistribution=dict(type="str", aliases=["static_route"]),
        connected_route_redistribution=dict(type="str", aliases=["connected_route"]),
        attached_host_route_redistribution=dict(type="str", aliases=["attached_host_route"]),
        ospf=dict(
            type="dict",
            options=dict(
                state=dict(type="str", choices=["enabled", "disabled"], default="enabled"),
                area_id=dict(type="str"),
                area_type=dict(type="str", choices=["regular", "stub", "nssa"]),
                cost=dict(type="int"),
                send_redistributed_lsas=dict(type="bool", aliases=["redistributed_lsas"]),
                originate_summary_lsa=dict(type="bool", aliases=["originate_lsa"]),
                suppress_forwarding_addr_translated_lsa=dict(type="bool", aliases=["suppress_fa_lsa"]),
                originate_default_route=dict(type="str", choices=list(ORIGINATE_DEFAULT_ROUTE)),
                originate_default_route_always=dict(type="bool", aliases=["always"]),
            ),
            required_if=[
                ["state", "enabled", ["area_id", "area_type"]],
            ],
        ),
        bgp=dict(
            type="dict",
            options=dict(
                state=dict(type="str", choices=["enabled", "disabled"], default="enabled"),
                inbound_route_map=dict(type="str", aliases=["import_route", "inbound_route"]),
                outbound_route_map=dict(type="str", aliases=["export_route", "outbound_route"]),
                route_dampening_ipv4=dict(type="str", aliases=["dampening_ipv4"]),
                route_dampening_ipv6=dict(type="str", aliases=["dampening_ipv6"]),
            ),
        ),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name", "uuid"], True],
            ["state", "present", ["name", "uuid"], True],
        ],
    )

    mso = MSOModule(module)

    l3out_template = module.params.get("l3out_template")
    name = module.params.get("name")
    uuid = module.params.get("uuid")
    description = module.params.get("description")
    l3_domain = module.params.get("l3_domain")
    pim = module.params.get("pim")
    interleak = module.params.get("interleak")
    static_route_redistribution = module.params.get("static_route_redistribution")
    connected_route_redistribution = module.params.get("connected_route_redistribution")
    attached_host_route_redistribution = module.params.get("attached_host_route_redistribution")
    target_dscp = TARGET_DSCP_MAP.get(module.params.get("target_dscp"))
    vrf_dict = module.params.get("vrf") if module.params.get("vrf") else {}
    ospf = module.params.get("ospf")
    bgp = module.params.get("bgp")

    state = module.params.get("state")

    mso_template = MSOTemplate(mso, "l3out", l3out_template)
    mso_template.validate_template("l3out")

    tenant_id = mso_template.template_summary.get("tenantId")
    tenant_name = mso_template.template_summary.get("tenantName")

    match = mso_template.get_l3out_object(uuid, name)

    if (uuid or name) and match:
        mso.existing = mso.previous = copy.deepcopy(insert_l3out_relation_name(match.details, mso_template))  # Query a specific object
    elif match:
        mso.existing = [insert_l3out_relation_name(l3out, mso_template) for l3out in match]  # Query all objects

    if state != "query":
        path = "/l3outTemplate/l3outs/{0}".format(match.index if match else "-")

    ops = []

    if state == "present":
        if uuid and not mso.existing:
            mso.fail_json(msg="L3Out with the uuid: '{0}' not found".format(uuid))

        templates_objects_path = "templates/objects"
        route_map_params = {"type": "routeMap", "tenant-id": tenant_id}
        route_map_path = generate_api_endpoint(templates_objects_path, **route_map_params)
        route_map_objects = mso.query_objs(route_map_path)

        vrf_ref = None
        if vrf_dict:
            vrf_object = mso_template.get_vrf_object(vrf_dict, tenant_id, templates_objects_path)
            if pim and vrf_object.details.get("l3MCast") is False:
                mso.fail_json(
                    msg="Invalid configuration in L3Out {0}, 'PIM' cannot be enabled while using the VRF '{1}' with L3 Multicast disabled".format(
                        "UUID: {0}".format(uuid) if uuid else "Name: {0}".format(name), vrf_dict.get("name")
                    )
                )
            vrf_ref = vrf_object.details.get("uuid")

        existing_routing_protocols = mso.existing.get("routingProtocol", None)

        ospf_state = ospf.get("state") if ospf else "ignore"
        bgp_state = bgp.get("state") if bgp else "ignore"

        res_routing_protocols = get_routing_protocol(None if existing_routing_protocols == "none" else existing_routing_protocols, ospf_state, bgp_state)

        routing_protocols = res_routing_protocols if res_routing_protocols else "none"

        if not mso.existing:
            payload = dict(name=name)

            payload["routingProtocol"] = routing_protocols

            if vrf_ref is None:
                mso.fail_json(msg="The O(vrf) is required during the creation.")

            payload["vrfRef"] = vrf_ref
            payload["vrfName"] = vrf_dict.get("name")

            if description:
                payload["description"] = description

            if l3_domain:
                payload["l3domain"] = l3_domain

            if target_dscp:
                payload["targetDscp"] = target_dscp

            if pim is not None:
                payload["pim"] = pim

            outer_route_maps = dict()
            if interleak:
                outer_route_maps["interleakName"] = interleak
                outer_route_maps["interleakRef"] = mso_template.get_route_map(
                    "interleak",
                    tenant_id,
                    tenant_name,
                    interleak,
                    route_map_objects,
                ).get("uuid", "")

            if static_route_redistribution:
                outer_route_maps["staticRouteRedistName"] = static_route_redistribution
                outer_route_maps["staticRouteRedistRef"] = mso_template.get_route_map(
                    "static_route_redistribution",
                    tenant_id,
                    tenant_name,
                    static_route_redistribution,
                    route_map_objects,
                ).get("uuid", "")

            if connected_route_redistribution:
                outer_route_maps["connectedRouteRedistName"] = connected_route_redistribution
                outer_route_maps["connectedRouteRedistRef"] = mso_template.get_route_map(
                    "connected_route_redistribution",
                    tenant_id,
                    tenant_name,
                    connected_route_redistribution,
                    route_map_objects,
                ).get("uuid", "")

            if attached_host_route_redistribution:
                outer_route_maps["attachedHostRouteRedistName"] = attached_host_route_redistribution
                outer_route_maps["attachedHostRouteRedistRef"] = mso_template.get_route_map(
                    "attached_host_route_redistribution",
                    tenant_id,
                    tenant_name,
                    attached_host_route_redistribution,
                    route_map_objects,
                ).get("uuid", "")

            if outer_route_maps:
                payload["advancedRouteMapRefs"] = outer_route_maps

            if bgp:
                if bgp.get("inbound_route_map"):
                    payload["importRouteMapName"] = bgp.get("inbound_route_map")
                    payload["importRouteMapRef"] = mso_template.get_route_map(
                        "inbound_route_map",
                        tenant_id,
                        tenant_name,
                        bgp.get("inbound_route_map"),
                        route_map_objects,
                    ).get("uuid", "")

                payload["importRouteControl"] = True if payload.get("importRouteMapRef") else False

                if bgp.get("outbound_route_map"):
                    payload["exportRouteMapName"] = bgp.get("outbound_route_map")
                    payload["exportRouteMapRef"] = mso_template.get_route_map(
                        "outbound_route_map",
                        tenant_id,
                        tenant_name,
                        bgp.get("outbound_route_map"),
                        route_map_objects,
                    ).get("uuid", "")

                if bgp.get("route_dampening_ipv4"):
                    payload["advancedRouteMapRefs"]["routeDampeningV4Name"] = bgp.get("route_dampening_ipv4")
                    payload["advancedRouteMapRefs"]["routeDampeningV4Ref"] = mso_template.get_route_map(
                        "route_dampening_ipv4",
                        tenant_id,
                        tenant_name,
                        bgp.get("route_dampening_ipv4"),
                        route_map_objects,
                    ).get("uuid", "")

                if bgp.get("route_dampening_ipv6"):
                    payload["advancedRouteMapRefs"]["routeDampeningV6Name"] = bgp.get("route_dampening_ipv6")
                    payload["advancedRouteMapRefs"]["routeDampeningV6Ref"] = mso_template.get_route_map(
                        "route_dampening_ipv6",
                        tenant_id,
                        tenant_name,
                        bgp.get("route_dampening_ipv6"),
                        route_map_objects,
                    ).get("uuid", "")

            if ospf:
                payload["ospfAreaConfig"] = dict(
                    cost=ospf.get("cost"),
                    id=ospf.get("area_id"),
                    areaType=ospf.get("area_type"),
                )

                default_route_leak = dict()
                if ospf.get("originate_default_route"):
                    default_route_leak["originateDefaultRoute"] = ORIGINATE_DEFAULT_ROUTE.get(ospf.get("originate_default_route"))

                if ospf.get("originate_default_route_always") is not None:
                    default_route_leak["always"] = ospf.get("originate_default_route_always")

                if default_route_leak:
                    payload["defaultRouteLeak"] = default_route_leak

                redistribute = ospf.get("send_redistributed_lsas")
                originate = ospf.get("originate_summary_lsa")
                suppress_fa = ospf.get("suppress_forwarding_addr_translated_lsa")

                control = dict()
                if redistribute is not None:
                    control["redistribute"] = redistribute

                if originate is not None:
                    control["originate"] = originate

                if suppress_fa is not None:
                    control["suppressFA"] = suppress_fa

                if control:
                    payload["ospfAreaConfig"]["control"] = control

            mso.sanitize(payload)
            ops.append(dict(op="add", path=path, value=payload))
        elif mso.existing:
            proposed_payload = copy.deepcopy(match.details)

            if name is not [None, ""] and mso.existing.get("name") != name:
                ops.append(dict(op="replace", path=path + "/name", value=name))
                proposed_payload["name"] = name

            if mso.existing.get("routingProtocol") != routing_protocols:
                ops.append(dict(op="replace", path=path + "/routingProtocol", value=routing_protocols))
                proposed_payload["routingProtocol"] = routing_protocols

            if vrf_ref is not None and mso.existing.get("vrfRef") != vrf_ref:
                ops.append(dict(op="replace", path=path + "/vrfRef", value=vrf_ref))
                proposed_payload["vrfRef"] = vrf_ref
                proposed_payload["vrfName"] = vrf_dict.get("name")

            if description is not None and mso.existing.get("description") != description:
                ops.append(dict(op="replace", path=path + "/description", value=description))
                proposed_payload["description"] = description

            if l3_domain is not None and mso.existing.get("l3domain") != l3_domain:
                ops.append(dict(op="replace", path=path + "/l3domain", value=l3_domain))
                proposed_payload["l3domain"] = l3_domain

            if target_dscp is not None and mso.existing.get("targetDscp") != target_dscp:
                ops.append(dict(op="replace", path=path + "/targetDscp", value=target_dscp))
                proposed_payload["targetDscp"] = target_dscp

            if pim is not None and mso.existing.get("pim") != pim:
                ops.append(dict(op="replace", path=path + "/pim", value=pim))
                proposed_payload["pim"] = pim

            if (
                interleak is not None
                or static_route_redistribution is not None
                or connected_route_redistribution is not None
                or attached_host_route_redistribution is not None
                or (bgp is not None and (bgp.get("route_dampening_ipv4") is not None or bgp.get("route_dampening_ipv6") is not None))
            ) and not mso.existing.get("advancedRouteMapRefs"):
                ops.append(dict(op="add", path=path + "/advancedRouteMapRefs", value=dict()))
                proposed_payload["advancedRouteMapRefs"] = dict()

            outer_route_maps = dict()

            if interleak is not None:
                interleak_ref = mso_template.get_route_map(
                    "interleak",
                    tenant_id,
                    tenant_name,
                    interleak,
                    route_map_objects,
                ).get("uuid", "")

                if mso.existing.get("advancedRouteMapRefs", {}).get("interleakRef") != interleak_ref:
                    ops.append(dict(op="replace", path=path + "/advancedRouteMapRefs/interleakRef", value=interleak_ref))
                    outer_route_maps["interleakRef"] = interleak_ref
                    outer_route_maps["interleakName"] = interleak

            if static_route_redistribution is not None:
                static_route_redistribution_ref = mso_template.get_route_map(
                    "static_route_redistribution",
                    tenant_id,
                    tenant_name,
                    static_route_redistribution,
                    route_map_objects,
                ).get("uuid", "")

                if mso.existing.get("advancedRouteMapRefs", {}).get("staticRouteRedistRef") != static_route_redistribution_ref:
                    ops.append(dict(op="replace", path=path + "/advancedRouteMapRefs/staticRouteRedistRef", value=static_route_redistribution_ref))
                    outer_route_maps["staticRouteRedistRef"] = static_route_redistribution_ref
                    outer_route_maps["staticRouteRedistName"] = static_route_redistribution

            if connected_route_redistribution is not None:
                connected_route_redistribution_ref = mso_template.get_route_map(
                    "connected_route_redistribution",
                    tenant_id,
                    tenant_name,
                    connected_route_redistribution,
                    route_map_objects,
                ).get("uuid", "")

                if mso.existing.get("advancedRouteMapRefs", {}).get("connectedRouteRedistRef") != connected_route_redistribution_ref:
                    ops.append(dict(op="replace", path=path + "/advancedRouteMapRefs/connectedRouteRedistRef", value=connected_route_redistribution_ref))
                    outer_route_maps["connectedRouteRedistRef"] = connected_route_redistribution_ref
                    outer_route_maps["connectedRouteRedistName"] = connected_route_redistribution

            if attached_host_route_redistribution is not None:
                attached_host_route_redistribution_ref = mso_template.get_route_map(
                    "attached_host_route_redistribution",
                    tenant_id,
                    tenant_name,
                    attached_host_route_redistribution,
                    route_map_objects,
                ).get("uuid", "")

                if mso.existing.get("advancedRouteMapRefs", {}).get("attachedHostRouteRedistRef") != attached_host_route_redistribution_ref:
                    ops.append(
                        dict(
                            op="replace",
                            path=path + "/advancedRouteMapRefs/attachedHostRouteRedistRef",
                            value=attached_host_route_redistribution_ref,
                        )
                    )
                    outer_route_maps["attachedHostRouteRedistRef"] = attached_host_route_redistribution_ref
                    outer_route_maps["attachedHostRouteRedistName"] = attached_host_route_redistribution

            if bgp and bgp_state != "disabled":
                if bgp.get("inbound_route_map") is not None:
                    inbound_route_map_ref = mso_template.get_route_map(
                        "inbound_route_map",
                        tenant_id,
                        tenant_name,
                        bgp.get("inbound_route_map"),
                        route_map_objects,
                    ).get("uuid", "")

                    if mso.existing.get("importRouteMapRef") != inbound_route_map_ref:
                        ops.append(dict(op="replace", path=path + "/importRouteMapRef", value=inbound_route_map_ref))
                        ops.append(dict(op="replace", path=path + "/importRouteControl", value=True if inbound_route_map_ref else False))

                        proposed_payload["importRouteMapRef"] = inbound_route_map_ref
                        proposed_payload["importRouteMapName"] = bgp.get("inbound_route_map")
                        proposed_payload["importRouteControl"] = True if inbound_route_map_ref else False

                if bgp.get("outbound_route_map") is not None:
                    outbound_route_map_ref = mso_template.get_route_map(
                        "outbound_route_map",
                        tenant_id,
                        tenant_name,
                        bgp.get("outbound_route_map"),
                        route_map_objects,
                    ).get("uuid", "")

                    if mso.existing.get("exportRouteMapRef") != outbound_route_map_ref:
                        ops.append(dict(op="replace", path=path + "/exportRouteMapRef", value=outbound_route_map_ref))
                        proposed_payload["exportRouteMapRef"] = outbound_route_map_ref
                        proposed_payload["exportRouteMapName"] = bgp.get("outbound_route_map")

                if bgp.get("route_dampening_ipv4") is not None:
                    route_dampening_ipv4_ref = mso_template.get_route_map(
                        "route_dampening_ipv4",
                        tenant_id,
                        tenant_name,
                        bgp.get("route_dampening_ipv4"),
                        route_map_objects,
                    ).get("uuid", "")

                    if mso.existing.get("advancedRouteMapRefs", {}).get("routeDampeningV4Ref") != route_dampening_ipv4_ref:
                        ops.append(dict(op="replace", path=path + "/advancedRouteMapRefs/routeDampeningV4Ref", value=route_dampening_ipv4_ref))
                        outer_route_maps["routeDampeningV4Ref"] = route_dampening_ipv4_ref
                        outer_route_maps["routeDampeningV4Name"] = bgp.get("route_dampening_ipv4")

                if bgp.get("route_dampening_ipv6") is not None:
                    route_dampening_ipv6_ref = mso_template.get_route_map(
                        "route_dampening_ipv6",
                        tenant_id,
                        tenant_name,
                        bgp.get("route_dampening_ipv6"),
                        route_map_objects,
                    ).get("uuid", "")

                    if mso.existing.get("advancedRouteMapRefs", {}).get("routeDampeningV6Ref") != route_dampening_ipv6_ref:
                        ops.append(dict(op="replace", path=path + "/advancedRouteMapRefs/routeDampeningV6Ref", value=route_dampening_ipv6_ref))
                        outer_route_maps["routeDampeningV6Ref"] = route_dampening_ipv6_ref
                        outer_route_maps["routeDampeningV6Name"] = bgp.get("route_dampening_ipv6")

            elif bgp_state == "disabled":
                ops.append(dict(op="replace", path=path + "/importRouteMapRef", value=""))
                ops.append(dict(op="replace", path=path + "/importRouteControl", value=False))
                proposed_payload.pop("importRouteMapRef", None)
                proposed_payload["importRouteControl"] = False

                ops.append(dict(op="replace", path=path + "/exportRouteMapRef", value=""))
                proposed_payload.pop("exportRouteMapRef", None)

                ops.append(dict(op="replace", path=path + "/advancedRouteMapRefs/routeDampeningV6Ref", value=""))
                proposed_payload.get("advancedRouteMapRefs", {}).pop("routeDampeningV6Ref", None)

                ops.append(dict(op="replace", path=path + "/advancedRouteMapRefs/routeDampeningV4Ref", value=""))
                proposed_payload.get("advancedRouteMapRefs", {}).pop("routeDampeningV4Ref", None)

            if outer_route_maps:
                for key, value in outer_route_maps.items():
                    proposed_payload["advancedRouteMapRefs"][key] = value

            if ospf and ospf_state != "disabled":
                originate_default_route = ORIGINATE_DEFAULT_ROUTE.get(ospf.get("originate_default_route"))
                originate_default_route_always = ospf.get("originate_default_route_always")

                if originate_default_route is not None and originate_default_route == "" and mso.existing.get("defaultRouteLeak"):
                    ops.append(dict(op="remove", path=path + "/defaultRouteLeak"))
                    proposed_payload.pop("defaultRouteLeak", None)

                elif (originate_default_route is not None and originate_default_route != "") or (originate_default_route_always is not None):
                    if not mso.existing.get("defaultRouteLeak"):
                        ops.append(dict(op="replace", path=path + "/defaultRouteLeak", value=dict()))
                        proposed_payload["defaultRouteLeak"] = dict()

                    if originate_default_route != mso.existing.get("defaultRouteLeak", {}).get("originateDefaultRoute"):
                        ops.append(
                            dict(
                                op="replace",
                                path=path + "/defaultRouteLeak/originateDefaultRoute",
                                value=originate_default_route,
                            )
                        )
                        proposed_payload["defaultRouteLeak"]["originateDefaultRoute"] = originate_default_route

                    if originate_default_route_always is not None and originate_default_route_always != mso.existing.get("defaultRouteLeak", {}).get("always"):
                        ops.append(dict(op="replace", path=path + "/defaultRouteLeak/always", value=originate_default_route_always))
                        proposed_payload["defaultRouteLeak"]["always"] = originate_default_route_always

                if not mso.existing.get("ospfAreaConfig"):
                    ops.append(dict(op="replace", path=path + "/ospfAreaConfig", value=dict()))
                    proposed_payload["ospfAreaConfig"] = dict()

                if ospf.get("cost") is not None and ospf.get("cost") != mso.existing.get("ospfAreaConfig", {}).get("cost"):
                    ops.append(dict(op="replace", path=path + "/ospfAreaConfig/cost", value=ospf.get("cost")))
                    proposed_payload["ospfAreaConfig"]["cost"] = ospf.get("cost")

                if ospf.get("area_id") is not None and ospf.get("area_id") != mso.existing.get("ospfAreaConfig", {}).get("id"):
                    ops.append(dict(op="replace", path=path + "/ospfAreaConfig/id", value=ospf.get("area_id")))
                    proposed_payload["ospfAreaConfig"]["id"] = ospf.get("area_id")

                if ospf.get("area_type") is not None and ospf.get("area_type") != mso.existing.get("ospfAreaConfig", {}).get("areaType"):
                    ops.append(dict(op="replace", path=path + "/ospfAreaConfig/areaType", value=ospf.get("area_type")))
                    proposed_payload["ospfAreaConfig"]["areaType"] = ospf.get("area_type")

                redistribute = ospf.get("send_redistributed_lsas")
                originate = ospf.get("originate_summary_lsa")
                suppress_fa = ospf.get("suppress_forwarding_addr_translated_lsa")

                if (redistribute is not None or originate is not None or suppress_fa is not None) and not mso.existing.get("ospfAreaConfig", {}).get(
                    "control"
                ):
                    ops.append(dict(op="replace", path=path + "/ospfAreaConfig/control", value=dict()))
                    proposed_payload["ospfAreaConfig"]["control"] = dict()

                if redistribute is not None and redistribute != mso.existing.get("ospfAreaConfig", {}).get("control", {}).get("redistribute"):
                    ops.append(dict(op="replace", path=path + "/ospfAreaConfig/control/redistribute", value=redistribute))
                    proposed_payload["ospfAreaConfig"]["control"]["redistribute"] = redistribute

                if originate is not None and originate != mso.existing.get("ospfAreaConfig", {}).get("control", {}).get("originate"):
                    ops.append(dict(op="replace", path=path + "/ospfAreaConfig/control/originate", value=originate))
                    proposed_payload["ospfAreaConfig"]["control"]["originate"] = originate

                if suppress_fa is not None and suppress_fa != mso.existing.get("ospfAreaConfig", {}).get("control", {}).get("suppressFA"):
                    ops.append(dict(op="replace", path=path + "/ospfAreaConfig/control/suppressFA", value=suppress_fa))
                    proposed_payload["ospfAreaConfig"]["control"]["suppressFA"] = suppress_fa

            elif ospf_state == "disabled":
                ops.append(dict(op="remove", path=path + "/ospfAreaConfig"))
                proposed_payload.pop("ospfAreaConfig", None)

                ops.append(dict(op="remove", path=path + "/defaultRouteLeak"))
                proposed_payload.pop("defaultRouteLeak", None)

            mso.sanitize(proposed_payload, collate=True)

    elif state == "absent":
        if mso.existing:
            ops.append(dict(op="remove", path="/l3outTemplate/l3outs/{0}".format(match.index)))

    if not module.check_mode and ops:
        response_object = mso.request(mso_template.template_path, method="PATCH", data=ops)

        mso_template.template = response_object
        match = mso_template.get_l3out_object(uuid, name)

        if match:
            mso.existing = insert_l3out_relation_name(match.details, mso_template)  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = insert_l3out_relation_name(mso.proposed, mso_template, module.check_mode) if state == "present" else {}

    mso.exit_json()


def insert_l3out_relation_name(l3out_object, mso_template, check_mode=False):
    l3out_object["vrfName"] = mso_template.get_template_object_name_by_uuid("vrf", l3out_object.get("vrfRef"))

    if not check_mode:
        l3out_relations = mso_template.mso.request("{0}/relations".format(mso_template.template_path), "GET")
        route_map_policies = l3out_relations.get("relations", {}).get("identities", {}).get("routeMapPolicies", [])

    if "exportRouteMapRef" in l3out_object:
        if check_mode:
            l3out_object["exportRouteMapName"] = mso_template.get_template_object_name_by_uuid("routeMap", l3out_object["exportRouteMapRef"])
        else:
            l3out_object["exportRouteMapName"] = (
                mso_template.get_object_by_key_value_pairs(
                    "Route Map",
                    route_map_policies,
                    [KVPair("ref", l3out_object["exportRouteMapRef"])],
                    True,
                ).details
                or {}
            ).get("displayName")

    if "importRouteMapRef" in l3out_object:
        if check_mode:
            l3out_object["importRouteMapName"] = mso_template.get_template_object_name_by_uuid("routeMap", l3out_object["importRouteMapRef"])
        else:
            l3out_object["importRouteMapName"] = (
                mso_template.get_object_by_key_value_pairs(
                    "Route Map",
                    route_map_policies,
                    [KVPair("ref", l3out_object["importRouteMapRef"])],
                    True,
                ).details
                or {}
            ).get("displayName")

    advanced_route_maps = l3out_object.get("advancedRouteMapRefs")
    if advanced_route_maps is not None:
        if "attachedHostRouteRedistRef" in advanced_route_maps:
            if check_mode:
                l3out_object["advancedRouteMapRefs"]["attachedHostRouteRedistName"] = mso_template.get_template_object_name_by_uuid(
                    "routeMap", advanced_route_maps["attachedHostRouteRedistRef"]
                )
            else:
                l3out_object["advancedRouteMapRefs"]["attachedHostRouteRedistName"] = (
                    mso_template.get_object_by_key_value_pairs(
                        "Route Map",
                        route_map_policies,
                        [KVPair("ref", advanced_route_maps["attachedHostRouteRedistRef"])],
                        True,
                    ).details
                    or {}
                ).get("displayName")

        if "connectedRouteRedistRef" in advanced_route_maps:
            if check_mode:
                l3out_object["advancedRouteMapRefs"]["connectedRouteRedistName"] = mso_template.get_template_object_name_by_uuid(
                    "routeMap", advanced_route_maps["connectedRouteRedistRef"]
                )
            else:
                l3out_object["advancedRouteMapRefs"]["connectedRouteRedistName"] = (
                    mso_template.get_object_by_key_value_pairs(
                        "Route Map",
                        route_map_policies,
                        [KVPair("ref", advanced_route_maps["connectedRouteRedistRef"])],
                        True,
                    ).details
                    or {}
                ).get("displayName")

        if "interleakRef" in advanced_route_maps:
            if check_mode:
                l3out_object["advancedRouteMapRefs"]["interleakName"] = mso_template.get_template_object_name_by_uuid(
                    "routeMap", advanced_route_maps["interleakRef"]
                )
            else:
                l3out_object["advancedRouteMapRefs"]["interleakName"] = (
                    mso_template.get_object_by_key_value_pairs(
                        "Route Map",
                        route_map_policies,
                        [KVPair("ref", advanced_route_maps["interleakRef"])],
                        True,
                    ).details
                    or {}
                ).get("displayName")

        if "routeDampeningV4Ref" in advanced_route_maps:
            if check_mode:
                l3out_object["advancedRouteMapRefs"]["routeDampeningV4Name"] = mso_template.get_template_object_name_by_uuid(
                    "routeMap", advanced_route_maps["routeDampeningV4Ref"]
                )
            else:
                l3out_object["advancedRouteMapRefs"]["routeDampeningV4Name"] = (
                    mso_template.get_object_by_key_value_pairs(
                        "Route Map",
                        route_map_policies,
                        [KVPair("ref", advanced_route_maps["routeDampeningV4Ref"])],
                        True,
                    ).details
                    or {}
                ).get("displayName")

        if "routeDampeningV6Ref" in advanced_route_maps:
            if check_mode:
                l3out_object["advancedRouteMapRefs"]["routeDampeningV6Name"] = mso_template.get_template_object_name_by_uuid(
                    "routeMap", advanced_route_maps["routeDampeningV6Ref"]
                )
            else:
                l3out_object["advancedRouteMapRefs"]["routeDampeningV6Name"] = (
                    mso_template.get_object_by_key_value_pairs(
                        "Route Map",
                        route_map_policies,
                        [KVPair("ref", advanced_route_maps["routeDampeningV6Ref"])],
                        True,
                    ).details
                    or {}
                ).get("displayName")

        if "staticRouteRedistRef" in advanced_route_maps:
            if check_mode:
                l3out_object["advancedRouteMapRefs"]["staticRouteRedistName"] = mso_template.get_template_object_name_by_uuid(
                    "routeMap", advanced_route_maps["staticRouteRedistRef"]
                )
            else:
                l3out_object["advancedRouteMapRefs"]["staticRouteRedistName"] = (
                    mso_template.get_object_by_key_value_pairs(
                        "Route Map",
                        route_map_policies,
                        [KVPair("ref", advanced_route_maps["staticRouteRedistRef"])],
                        True,
                    ).details
                    or {}
                ).get("displayName")
    return l3out_object


if __name__ == "__main__":
    main()



# (c) 2018 Red Hat Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
author: Ansible Networking Team (@ansible-network)
name: nxos
short_description: Use NX-API to run commands on Cisco NX-OS platform
description:
- This plugin provides low level abstraction APIs for sending and receiving
  commands using NX-API with devices running Cisco NX-OS.
version_added: 1.0.0
"""

import collections
import json
import re

from ansible.module_utils._text import to_text
from ansible.module_utils.connection import ConnectionError
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import to_list
from ansible_collections.ansible.netcommon.plugins.plugin_utils.httpapi_base import HttpApiBase


OPTIONS = {
    "format": ["text", "json"],
    "diff_match": ["line", "strict", "exact", "none"],
    "diff_replace": ["line", "block", "config"],
    "output": ["text", "json"],
}


class HttpApi(HttpApiBase):
    def __init__(self, *args, **kwargs):
        super(HttpApi, self).__init__(*args, **kwargs)
        self._device_info = None
        self._module_context = {}

    def read_module_context(self, module_key):
        if self._module_context.get(module_key):
            return self._module_context[module_key]

        return None

    def save_module_context(self, module_key, module_context):
        self._module_context[module_key] = module_context

        return None

    def send_request(self, data, **message_kwargs):
        output = None
        queue = list()
        responses = list()

        for item in to_list(data):
            cmd_output = message_kwargs.get("output") or "text"
            if isinstance(item, dict):
                command = item["command"]
                if "output" in item:
                    cmd_output = item["output"]
            else:
                command = item

            # Emulate '| json' from CLI
            if command.endswith("| json"):
                command = command.rsplit("|", 1)[0]
                cmd_output = "json"

            if output and output != cmd_output:
                responses.extend(self._run_queue(queue, output))
                queue = list()

            output = cmd_output
            queue.append(command)

        if queue:
            responses.extend(self._run_queue(queue, output))

        if len(responses) == 1:
            return responses[0]
        return responses

    def _run_queue(self, queue, output):
        if self._become:
            self.connection.queue_message(
                "warning",
                "become has no effect over httpapi. Use network_cli if privilege escalation is required",
            )

        request = request_builder(queue, output)
        headers = {"Content-Type": "application/json"}

        response, response_data = self.connection.send(
            "/ins",
            request,
            headers=headers,
            method="POST",
        )

        try:
            response_data = json.loads(to_text(response_data.getvalue()))
        except ValueError:
            raise ConnectionError(
                "Response was not valid JSON, got {0}".format(to_text(response_data.getvalue())),
            )

        results = handle_response(response_data)
        return results

    def get_device_info(self):
        if self._device_info:
            return self._device_info

        device_info = {}

        device_info["network_os"] = "nxos"
        reply, platform_reply = self.send_request(["show version", "show inventory"])

        find_os_version = [
            r"\s+system:\s+version\s*(\S+)",
            r"\s+kickstart:\s+version\s*(\S+)",
            r"\s+NXOS:\s+version\s*(\S+)",
        ]
        for regex in find_os_version:
            match_ver = re.search(regex, reply, re.M)
            if match_ver:
                device_info["network_os_version"] = match_ver.group(1)
                break

        match_chassis_id = re.search(r"Hardware\n\s+cisco\s*(\S+\s+\S+)", reply, re.M)
        if match_chassis_id:
            device_info["network_os_model"] = match_chassis_id.group(1)

        match_host_name = re.search(r"\s+Device name:\s*(\S+)", reply, re.M)
        if match_host_name:
            device_info["network_os_hostname"] = match_host_name.group(1)

        find_os_image = [
            r"\s+system image file is:\s*(\S+)",
            r"\s+kickstart image file is:\s*(\S+)",
            r"\s+NXOS image file is:\s*(\S+)",
        ]
        for regex in find_os_image:
            match_file_name = re.search(regex, reply, re.M)
            if match_file_name:
                device_info["network_os_image"] = match_file_name.group(1)
                break

        match_os_platform = re.search(
            r'NAME: (?:"Chassis"| Chassis ),\s*DESCR:.*\nPID:\s*(\S+)',
            platform_reply,
            re.M,
        )
        if match_os_platform:
            device_info["network_os_platform"] = match_os_platform.group(1)

        self._device_info = device_info
        return self._device_info

    def get_device_operations(self):
        platform = self.get_device_info().get("network_os_platform", "")
        return {
            "supports_diff_replace": True,
            "supports_commit": False,
            "supports_rollback": False,
            "supports_defaults": True,
            "supports_onbox_diff": False,
            "supports_commit_comment": False,
            "supports_multiline_delimiter": False,
            "supports_diff_match": True,
            "supports_diff_ignore_lines": True,
            "supports_generate_diff": True,
            "supports_replace": True if "9K" in platform else False,
        }

    def get_capabilities(self):
        result = {}
        result["rpc"] = []
        result["device_info"] = self.get_device_info()
        result["device_operations"] = self.get_device_operations()
        result.update(OPTIONS)
        result["network_api"] = "nxapi"

        return json.dumps(result)

    # Shims for resource module support
    def get(self, command, output=None):
        # This method is ONLY here to support resource modules. Therefore most
        # arguments are unsupported and not present.

        return self.send_request(data=command, output=output)

    def edit_config(self, candidate):
        # This method is ONLY here to support resource modules. Therefore most
        # arguments are unsupported and not present.

        responses = self.send_request(candidate, output="config")
        return [resp for resp in to_list(responses) if resp != "{}"]


def handle_response(response):
    results = []

    if response["ins_api"].get("outputs"):
        for output in to_list(response["ins_api"]["outputs"]["output"]):
            if output["code"] != "200":
                # Best effort messages: some API output keys may not exist on some platforms
                input_data = output.get("input", "")
                msg = output.get("msg", "")
                clierror = output.get("clierror", "")
                raise ConnectionError(
                    "%s: %s: %s" % (input_data, msg, clierror),
                    code=output["code"],
                )
            elif "body" in output:
                result = output["body"]
                if isinstance(result, dict):
                    result = json.dumps(result)

                results.append(result.strip())

    return results


def request_builder(commands, output, version="1.0", chunk="0", sid=None):
    """Encodes a NXAPI JSON request message"""
    output_to_command_type = {
        "text": "cli_show_ascii",
        "json": "cli_show",
        "bash": "bash",
        "config": "cli_conf",
    }

    maybe_output = commands[0].split("|")[-1].strip()
    if maybe_output in output_to_command_type:
        command_type = output_to_command_type[maybe_output]
        commands = [command.split("|")[0].strip() for command in commands]
    else:
        try:
            command_type = output_to_command_type[output]
        except KeyError:
            msg = "invalid format, received %s, expected one of %s" % (
                output,
                ",".join(output_to_command_type.keys()),
            )
            raise ConnectionError(msg)

    if isinstance(commands, (list, set, tuple)):
        commands = " ;".join(commands)

    # Order should not matter but some versions of NX-OS software fail
    # to process the payload properly if 'input' gets serialized before
    # 'type' and the payload of 'input' contains the word 'type'.
    msg = collections.OrderedDict()
    msg["version"] = version
    msg["type"] = command_type
    msg["chunk"] = chunk
    msg["sid"] = sid
    msg["input"] = commands
    msg["output_format"] = "json"

    return json.dumps(dict(ins_api=msg))



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat Inc.
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

##############################################
#                 WARNING
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#
##############################################

"""
The module file for ios_l3_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: iosxr_l3_interfaces
short_description: Resource module to configure L3 interfaces.
description: This module provides declarative management of Layer-3 interface on Cisco
  IOS-XR devices.
version_added: 1.0.0
author:
- Sumit Jaiswal (@justjais)
- Rohit Thakur (@rohitthakur2590)
notes:
- This module works with connection C(network_cli). See L(the IOS-XR Platform Options,../network/user_guide/platform_iosxr.html).
options:
  config:
    description: A dictionary of Layer-3 interface options
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - Full name of the interface excluding any logical unit number, i.e. GigabitEthernet0/1.
        type: str
        required: true
      ipv4:
        description:
        - IPv4 address to be set for the Layer-3 interface mentioned in I(name) option.
        - The address format is <ipv4 address>/<mask>, the mask is number in range
          0-32 eg. 192.168.0.1/24
        type: list
        elements: dict
        suboptions:
          address:
            description:
            - Configures the IPv4 address for Interface.
            type: str
          secondary:
            description:
            - Configures the IP address as a secondary address.
            type: bool
      ipv6:
        description:
        - IPv6 address to be set for the Layer-3 interface mentioned in I(name) option.
        - The address format is <ipv6 address>/<mask>, the mask is number in range
          0-128 eg. fd5d:12c9:2201:1::1/64
        type: list
        elements: dict
        suboptions:
          address:
            description:
            - Configures the IPv6 address for Interface.
            type: str
      carrier_delay:
        description:
        - Delay carrier transitions.
        type: dict
        suboptions:
          up:
            description:
            - Delay for carrier-up event in milliseconds.
            type: int
          down:
            description:
            - Delay for carrier-down event in milliseconds.
            type: int
      dampening:
        description:
        - Enable interface dampening to reduce the effects of interface flapping.
        type: dict
        suboptions:
          enabled:
            description:
            - Enable or disable dampening.
            type: bool
          half_life:
            description:
            - The time in minutes after which a penalty is halved if the interface stays up.
            type: int
          reuse_threshold:
            description:
            - The penalty value below which a suppressed interface is unsuppressed.
            type: int
          suppress_threshold:
            description:
            - The penalty value above which an interface is suppressed.
            type: int
          max_suppress_time:
            description:
            - The maximum time in minutes that an interface can be suppressed.
            type: int
          restart_penalty:
            description:
            - The penalty assigned to an interface when it flaps.
            type: int
      load_interval:
        description:
        - Specify the length of time for which data is used to compute load statistics.
        type: int
      flow_control:
        description:
        - Configure flow control for the interface.
        type: str
        choices: ['ingress', 'egress', 'bidirectional']
      flow:
        description:
        - Configure NetFlow monitoring on the interface.
        type: dict
        suboptions:
          ipv4:
            description:
            - Configure NetFlow monitoring for IPv4 traffic.
            type: dict
            suboptions:
              monitor:
                description:
                - Name of the flow monitor to apply.
                type: str
              sampler:
                description:
                - Name of the sampler to apply.
                type: str
              direction:
                description:
                - Direction of the flow monitoring.
                type: str
                choices: ['ingress', 'egress', 'bidirectional']
          ipv6:
            description:
            - Configure NetFlow monitoring for IPv6 traffic.
            type: dict
            suboptions:
              monitor:
                description:
                - Name of the flow monitor to apply.
                type: str
              sampler:
                description:
                - Name of the sampler to apply.
                type: str
              direction:
                description:
                - Direction of the flow monitoring.
                type: str
                choices: ['ingress', 'egress', 'bidirectional']
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the IOS-XR device
      by executing the command B(show running-config interface).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - parsed
    - rendered
    - gathered
    default: merged
    description:
    - The state of the configuration after module completion
    type: str
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# viosxr#show running-config interface
# interface Loopback888
# !
# interface Loopback999
# !
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface preconfigure GigabitEthernet0/0/0/2
# !
# interface preconfigure GigabitEthernet0/0/0/3
# !

- name: Merge provided configuration with device configuration
  cisco.iosxr.iosxr_l3_interfaces:
    config:
      - name: GigabitEthernet0/0/0/2
        ipv4:
          - address: 192.168.0.1/24
      - name: GigabitEthernet0/0/0/3
        ipv4:
          - address: 192.168.2.1/24
            secondary: true
    state: merged

# Task Output
# -----------
#
# before:
# - name: Loopback888
# - name: Loopback999
# - name: GigabitEthernet0/0/0/2
# - name: GigabitEthernet0/0/0/3
# commands:
# - interface GigabitEthernet0/0/0/2
# - ipv4 address 192.168.0.1 255.255.255.0
# - interface GigabitEthernet0/0/0/3
# - ipv4 address 192.168.2.1 255.255.255.0 secondary
# after:
# - name: Loopback888
# - name: Loopback999
# - ipv4:
#   - address: 192.168.0.1/24
#   name: GigabitEthernet0/0/0/2
# - ipv4:
#   - address: 192.168.2.1/24
#     secondary: true
#   name: GigabitEthernet0/0/0/3

# After state:
# ------------
#
# viosxr#show running-config interface
# interface Loopback888
# !
# interface Loopback999
# !
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface preconfigure GigabitEthernet0/0/0/2
#  ipv4 address 192.168.0.1 255.255.255.0
# !
# interface preconfigure GigabitEthernet0/0/0/3
#  ipv4 address 192.168.2.1 255.255.255.0 secondary
# !

# Using overridden

# Before state:
# -------------
#
# viosxr#show running-config interface
# interface Loopback888
# !
# interface Loopback999
# !
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface preconfigure GigabitEthernet0/0/0/2
#  ipv4 address 192.168.0.1 255.255.255.0
# !
# interface preconfigure GigabitEthernet0/0/0/3
#  ipv4 address 192.168.2.1 255.255.255.0 secondary
# !

- name: Override device configuration of all interfaces with provided configuration
  cisco.iosxr.iosxr_l3_interfaces:
    config:
      - name: GigabitEthernet0/0/0/3
        ipv4:
          - address: 192.168.0.2/24
          - address: 192.168.2.1/24
            secondary: true
    state: overridden


# Task Output
# -----------
#
# before:
# - name: Loopback888
# - name: Loopback999
# - ipv4:
#   - address: 192.168.0.1/24
#   name: GigabitEthernet0/0/0/2
# - ipv4:
#   - address: 192.168.2.1/24
#     secondary: true
#   name: GigabitEthernet0/0/0/3
# commands:
# - interface GigabitEthernet0/0/0/2
# - no ipv4 address
# - interface GigabitEthernet0/0/0/3
# - ipv4 address 192.168.0.2 255.255.255.0
# - ipv4 address 192.168.0.1 255.255.255.0
# after:
# - name: Loopback888
# - name: Loopback999
# - name: GigabitEthernet0/0/0/2
# - ipv4:
#   - address: 192.168.0.1/24
#   - address: 192.168.2.1/24
#     secondary: true
#   name: GigabitEthernet0/0/0/3

# After state:
# -------------
#
# viosxr#show running-config interface
# interface Loopback888
# !
# interface Loopback999
# !
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface preconfigure GigabitEthernet0/0/0/2
# !
# interface preconfigure GigabitEthernet0/0/0/3
#  ipv4 address 192.168.0.1 255.255.255.0
#  ipv4 address 192.168.2.1 255.255.255.0 secondary
# !

# Using replaced

# Before state:
# -------------
#
# viosxr#show running-config interface
# interface Loopback888
# !
# interface Loopback999
# !
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface preconfigure GigabitEthernet0/0/0/2
# !
# interface preconfigure GigabitEthernet0/0/0/3
#  ipv4 address 192.168.0.1 255.255.255.0
#  ipv4 address 192.168.2.1 255.255.255.0 secondary
# !

- name: >-
    Replaces device configuration of listed interfaces with provided
    configuration
  cisco.iosxr.iosxr_l3_interfaces:
    config:
      - name: GigabitEthernet0/0/0/3
        ipv6:
          - address: 'fd5d:12c9:2201:1::1/64'
      - name: GigabitEthernet0/0/0/2
        ipv4:
          - address: 192.168.0.2/24
    state: replaced


# Task Output
# -----------
#
# before:
# - name: Loopback888
# - name: Loopback999
# - name: GigabitEthernet0/0/0/2
# - ipv4:
#   - address: 192.168.0.1/24
#   - address: 192.168.2.1/24
#     secondary: true
#   name: GigabitEthernet0/0/0/3
# commands:
# - interface GigabitEthernet0/0/0/3
# - no ipv4 address
# - ipv6 address fd5d:12c9:2201:1::1/64
# - interface GigabitEthernet0/0/0/2
# - ipv4 address 192.168.0.2 255.255.255.0
# after:
# - name: Loopback888
# - name: Loopback999
# - ipv4:
#   - address: 192.168.0.2/24
#   name: GigabitEthernet0/0/0/2
# - ipv6:
#   - address: fd5d:12c9:2201:1::1/64
#   name: GigabitEthernet0/0/0/3

# After state:
# -------------
#
# viosxr#show running-config interface
# interface Loopback888
# !
# interface Loopback999
# !
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface preconfigure GigabitEthernet0/0/0/2
#  ipv4 address 192.168.0.2 255.255.255.0
# !
# interface preconfigure GigabitEthernet0/0/0/3
#  ipv6 address fd5d:12c9:2201:1::1/64
# !

# Using deleted

# Before state:
# -------------
#
# viosxr#show running-config interface
# interface Loopback888
# !
# interface Loopback999
# !
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface preconfigure GigabitEthernet0/0/0/2
#  ipv4 address 192.168.0.2 255.255.255.0
# !
# interface preconfigure GigabitEthernet0/0/0/3
#  ipv6 address fd5d:12c9:2201:1::1/64
# !

- name: Delete attributes for interfaces (This won't delete the interface itself)
  cisco.iosxr.iosxr_l3_interfaces:
    config:
      - name: GigabitEthernet0/0/0/3
      - name: GigabitEthernet0/0/0/4
      - name: GigabitEthernet0/0/0/3.700
    state: deleted


# Task Output
# -----------
#
# before:
# - name: Loopback888
# - name: Loopback999
# - ipv4:
#   - address: 192.168.0.2/24
#   name: GigabitEthernet0/0/0/2
# - ipv6:
#   - address: fd5d:12c9:2201:1::1/64
#   name: GigabitEthernet0/0/0/3
# commands:
# - interface GigabitEthernet0/0/0/3
# - no ipv6 address
# after:
# - name: Loopback888
# - name: Loopback999
# - ipv4:
#   - address: 192.168.0.2/24
#   name: GigabitEthernet0/0/0/2
# - name: GigabitEthernet0/0/0/3

# After state:
# -------------
#
# viosxr#show running-config interface
# interface Loopback888
# !
# interface Loopback999
# !
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface preconfigure GigabitEthernet0/0/0/2
#  ipv4 address 192.168.0.2 255.255.255.0
# !
# interface preconfigure GigabitEthernet0/0/0/3
# !

# Using deleted - will delete all interface configuration and not interface

# Before state:
# -------------
#
# viosxr#show running-config interface
# interface Loopback888
# !
# interface Loopback999
# !
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface preconfigure GigabitEthernet0/0/0/1
#  ipv4 address 192.168.2.1 255.255.255.0
#  shutdown
# !
# interface preconfigure GigabitEthernet0/0/0/2
#  ipv4 address 192.168.3.1 255.255.255.0
#  shutdown
# !
# interface preconfigure GigabitEthernet0/0/0/3
#  ipv4 address 192.168.0.2 255.255.255.0
#  shutdown
# !
# interface preconfigure GigabitEthernet0/0/0/3.700
#  ipv4 address 192.168.0.1 255.255.255.0
# !
# interface preconfigure GigabitEthernet0/0/0/4
#  ipv6 address fd5d:12c9:2201:1::1/64
#  shutdown
# !

- name: "Delete L3 config of all interfaces (This won't delete the interface itself)"
  cisco.iosxr.iosxr_l3_interfaces:
    state: deleted

# Task Output
# -----------
#
# before:
# - name: Loopback888
# - name: Loopback999
# - ipv4:
#   - address: 192.168.2.1/24
#   name: GigabitEthernet0/0/0/1
# - ipv4:
#   - address: 192.168.3.1/24
#   name: GigabitEthernet0/0/0/2
# - ipv4:
#   - address: 192.168.0.2/24
#   name: GigabitEthernet0/0/0/3
# - ipv4:
#   - address: 192.168.0.1/24
#   name: GigabitEthernet0/0/0/3.700
# - ipv6:
#   - address: fd5d:12c9:2201:1::1/64
#   name: GigabitEthernet0/0/0/4
# commands:
# - interface GigabitEthernet0/0/0/1
# - no ipv4 address
# - interface GigabitEthernet0/0/0/2
# - no ipv4 address
# - interface GigabitEthernet0/0/0/3
# - no ipv4 address
# - interface GigabitEthernet0/0/0/3.700
# - no ipv4 address
# - interface GigabitEthernet0/0/0/4
# - no ipv6 address
# after:
# - name: Loopback888
# - name: Loopback999
# - name: GigabitEthernet0/0/0/1
# - name: GigabitEthernet0/0/0/2
# - name: GigabitEthernet0/0/0/3
# - name: GigabitEthernet0/0/0/3.700
# - name: GigabitEthernet0/0/0/4

# After state:
# -------------
#
# viosxr#show running-config interface
# interface Loopback888
# !
# interface Loopback999
# !
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface preconfigure GigabitEthernet0/0/0/1
#  shutdown
# !
# interface preconfigure GigabitEthernet0/0/0/2
#  shutdown
# !
# interface preconfigure GigabitEthernet0/0/0/3
#  shutdown
# !
# interface preconfigure GigabitEthernet0/0/0/3.700
# !
# interface preconfigure GigabitEthernet0/0/0/4
#  shutdown
# !


# Using parsed

# File: parsed.cfg
# ----------------
#
# interface Loopback888
#  description test for ansible
#  shutdown
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 10.8.38.70 255.255.255.0
# !
# interface GigabitEthernet0/0/0/0
#  description Configured and Merged by Ansible-Network
#  mtu 66
#  ipv4 address 192.0.2.1 255.255.255.0
#  ipv4 address 192.0.2.2 255.255.255.0 secondary
#  ipv6 address 2001:db8:0:3::/64
#  duplex half
# !
# interface GigabitEthernet0/0/0/1
#  description Configured and Merged by Ansible-Network
#  mtu 66
#  speed 100
#  duplex full
#  dot1q native vlan 10
#  l2transport
#   l2protocol cdp forward
#   l2protocol pvst tunnel
#   propagate remote-status
#  !
# !
# interface GigabitEthernet0/0/0/3
#  ipv4 address 192.0.22.1 255.255.255.0
#  ipv4 address 192.0.23.1 255.255.255.0
# !

- name: Convert L3 interfaces config to argspec without connecting to the appliance
  cisco.iosxr.iosxr_l3_interfaces:
    running_config: "{{ lookup('file', './parsed.cfg') }}"
    state: parsed

# Task Output
# -----------
#
# parsed:
# - ipv4:
#   - address: 192.0.2.1 255.255.255.0
#   - address: 192.0.2.2 255.255.255.0
#     secondary: true
#   ipv6:
#   - address: 2001:db8:0:3::/64
#   name: GigabitEthernet0/0/0/0
# - name: GigabitEthernet0/0/0/1
# - ipv4:
#   - address: 192.0.22.1 255.255.255.0
#   - address: 192.0.23.1 255.255.255.0
#   name: GigabitEthernet0/0/0/3

# Using rendered

- name: Render platform specific commands from task input using rendered state
  cisco.iosxr.iosxr_l3_interfaces:
    config:
      - name: GigabitEthernet0/0/0/0
        ipv4:
          - address: 198.51.100.1/24
      - name: GigabitEthernet0/0/0/1
        ipv6:
          - address: '2001:db8:0:3::/64'
        ipv4:
          - address: 192.0.2.1/24
          - address: 192.0.2.2/24
            secondary: true
    state: rendered


# Task Output
# -----------
#
# rendered:
# - interface GigabitEthernet0/0/0/0
# - ipv4 address 198.51.100.1 255.255.255.0
# - interface GigabitEthernet0/0/0/1
# - ipv4 address 192.0.2.2 255.255.255.0 secondary
# - ipv4 address 192.0.2.1 255.255.255.0
# - ipv6 address 2001:db8:0:3::/64

# Using gathered

# Before state:
# ------------
#
# RP/0/0/CPU0:an-iosxr-02#show running-config  interface
# interface Loopback888
#  description test for ansible
#  shutdown
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 10.8.38.70 255.255.255.0
# !
# interface GigabitEthernet0/0/0/0
#  description Configured and Merged by Ansible-Network
#  mtu 66
#  ipv4 address 192.0.2.1 255.255.255.0
#  ipv4 address 192.0.2.2 255.255.255.0 secondary
#  ipv6 address 2001:db8:0:3::/64
#  duplex half
# !
# interface GigabitEthernet0/0/0/1
#  description Configured and Merged by Ansible-Network
#  mtu 66
#  speed 100
#  duplex full
#  dot1q native vlan 10
#  l2transport
#   l2protocol cdp forward
#   l2protocol pvst tunnel
#   propagate remote-status
#  !
# !
# interface GigabitEthernet0/0/0/3
#  shutdown
# !
# interface GigabitEthernet0/0/0/4
#  shutdown
#  dot1q native vlan 40
# !

- name: Gather l3 interfaces facts
  cisco.iosxr.iosxr_l3_interfaces:
    config:
    state: gathered

# Task Output
# -----------
#
# gathered:
# - name: Loopback888
# - ipv4:
#   - address: 192.0.2.1 255.255.255.0
#   - address: 192.0.2.2 255.255.255.0
#     secondary: true
#   ipv6:
#   - address: 2001:db8:0:3::/64
#   name: GigabitEthernet0/0/0/0
# - name: GigabitEthernet0/0/0/1
# - name: GigabitEthernet0/0/0/3
# - name: GigabitEthernet0/0/0/4

# Using bidirectional flow monitoring (merged)

# Before state:
# -------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/0
#  ipv4 address 198.51.100.1 255.255.255.0
# !

- name: Configure bidirectional NetFlow on an interface (merged)
  cisco.iosxr.iosxr_l3_interfaces:
    config:
      - name: GigabitEthernet0/0/0/0
        flow:
          ipv4:
            monitor: FlowMap-IPv4
            sampler: NETFLOW_1
            direction: bidirectional
          ipv6:
            monitor: FlowMap-IPv6
            sampler: NETFLOW_1
            direction: bidirectional
    state: merged

# Task Output
# -----------
#
# before:
# - name: GigabitEthernet0/0/0/0
#   ipv4:
#   - address: 198.51.100.1/24
# commands:
# - interface GigabitEthernet0/0/0/0
# - flow ipv4 monitor FlowMap-IPv4 sampler NETFLOW_1 ingress
# - flow ipv4 monitor FlowMap-IPv4 sampler NETFLOW_1 egress
# - flow ipv6 monitor FlowMap-IPv6 sampler NETFLOW_1 ingress
# - flow ipv6 monitor FlowMap-IPv6 sampler NETFLOW_1 egress
# after:
# - name: GigabitEthernet0/0/0/0
#   ipv4:
#   - address: 198.51.100.1/24
#   flow:
#     ipv4:
#       monitor: FlowMap-IPv4
#       sampler: NETFLOW_1
#       direction: bidirectional
#     ipv6:
#       monitor: FlowMap-IPv6
#       sampler: NETFLOW_1
#       direction: bidirectional

# After state:
# ------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/0
#  ipv4 address 198.51.100.1 255.255.255.0
#  flow ipv4 monitor FlowMap-IPv4 sampler NETFLOW_1 ingress
#  flow ipv4 monitor FlowMap-IPv4 sampler NETFLOW_1 egress
#  flow ipv6 monitor FlowMap-IPv6 sampler NETFLOW_1 ingress
#  flow ipv6 monitor FlowMap-IPv6 sampler NETFLOW_1 egress
# !

# Using bidirectional flow monitoring (deleted)

# Before state:
# -------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/0
#  ipv4 address 198.51.100.1 255.255.255.0
#  flow ipv4 monitor FlowMap-IPv4 sampler NETFLOW_1 ingress
#  flow ipv4 monitor FlowMap-IPv4 sampler NETFLOW_1 egress
#  flow ipv6 monitor FlowMap-IPv6 sampler NETFLOW_1 ingress
#  flow ipv6 monitor FlowMap-IPv6 sampler NETFLOW_1 egress
# !

- name: >-
    Remove L3 attributes for an interface including bidirectional flow
    (does not remove the interface itself)
  cisco.iosxr.iosxr_l3_interfaces:
    config:
      - name: GigabitEthernet0/0/0/0
    state: deleted

# Task Output
# -----------
#
# before:
# - name: GigabitEthernet0/0/0/0
#   ipv4:
#   - address: 198.51.100.1/24
#   flow:
#     ipv4:
#       monitor: FlowMap-IPv4
#       sampler: NETFLOW_1
#       direction: bidirectional
#     ipv6:
#       monitor: FlowMap-IPv6
#       sampler: NETFLOW_1
#       direction: bidirectional
# commands:
# - interface GigabitEthernet0/0/0/0
# - no ipv4 address
# - no flow ipv4 monitor FlowMap-IPv4 sampler NETFLOW_1 ingress
# - no flow ipv4 monitor FlowMap-IPv4 sampler NETFLOW_1 egress
# - no flow ipv6 monitor FlowMap-IPv6 sampler NETFLOW_1 ingress
# - no flow ipv6 monitor FlowMap-IPv6 sampler NETFLOW_1 egress
# after:
# - name: GigabitEthernet0/0/0/0

# After state:
# ------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/0
# !
"""

RETURN = """
before:
  description: The configuration as structured data prior to module invocation.
  returned: always
  type: list
  sample: The configuration returned will always be in the same format of the parameters above.
after:
  description: The configuration as structured data after module completion.
  returned: when changed
  type: list
  sample: The configuration returned will always be in the same format of the parameters above.
commands:
  description: The set of commands pushed to the remote device
  returned: always
  type: list
  sample: ['interface GigabitEthernet0/0/0/1', 'ipv4 address 192.168.0.1 255.255.255.0']
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.l3_interfaces.l3_interfaces import (
    L3_InterfacesArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.l3_interfaces.l3_interfaces import (
    L3_Interfaces,
)


def main():
    """
    Main entry point for module execution
    :returns: the result form module invocation
    """
    required_if = [
        ("state", "merged", ("config",)),
        ("state", "replaced", ("config",)),
        ("state", "rendered", ("config",)),
        ("state", "overridden", ("config",)),
        ("state", "parsed", ("running_config",)),
    ]

    mutually_exclusive = [("config", "running_config")]
    module = AnsibleModule(
        argument_spec=L3_InterfacesArgs.argument_spec,
        required_if=required_if,
        supports_check_mode=True,
        mutually_exclusive=mutually_exclusive,
    )

    result = L3_Interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for nxos_vrf_address_family
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: nxos_vrf_address_family
short_description: Resource module to configure VRF address family definitions.
description: This module provides declarative management of VRF definitions on Cisco NXOS.
version_added: 9.3.0
author: Vinay Mulugund (@roverflow)
notes:
  - Tested against NX-OS 9.3.6.
  - This module works with connection C(network_cli) and C(httpapi).
    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_nxos.html)
options:
  config:
    description: A list of device configurations for VRF address family.
    type: list
    elements: dict
    suboptions:
      name:
        description: Name of the VRF.
        type: str
        required: true
      address_families:
        description: Enable address family and enter its config mode - AFI/SAFI configuration
        type: list
        elements: dict
        suboptions:
          afi:
            description: Address Family Identifier (AFI)
            type: str
            choices: ["ipv4", "ipv6"]
          safi:
            description: Address Family modifier
            type: str
            choices: ["multicast", "unicast"]
          maximum:
            description: Set a limit of routes
            type: dict
            suboptions:
              max_routes:
                description: Maximum number of routes allowed
                type: int
              max_route_options:
                description: Configure the options for maximum routes
                type: dict
                suboptions:
                  warning_only:
                    description: Configure only give a warning message if limit is exceeded
                    type: bool
                  threshold:
                    description: Configure threshold & its options
                    type: dict
                    suboptions:
                      threshold_value:
                        description: Threshold value (%) at which to generate a warning msg
                        type: int
                      reinstall_threshold:
                        description: Threshold value (%) at which to reinstall routes back to VRF
                        type: int
          route_target:
            description: Specify Target VPN Extended Communities
            type: list
            elements: dict
            suboptions:
              import:
                description: Import Target-VPN community
                type: str
              export:
                description: Export Target-VPN community
                type: str
          export:
            description: VRF export
            type: list
            elements: dict
            suboptions:
              map:
                description: Route-map based VRF export
                type: str
              vrf:
                description: Virtual Router Context
                type: dict
                suboptions:
                  max_prefix:
                    description: Maximum prefix limit
                    type: int
                  map_import:
                    description: Route-map based VRF import
                    type: str
                  allow_vpn:
                    description: Allow re-importation of VPN imported routes
                    type: bool
          import:
            description: VRF import
            type: list
            elements: dict
            suboptions:
              map:
                description: Route-map based VRF export
                type: str
              vrf:
                description: Virtual Router Context
                type: dict
                suboptions:
                  max_prefix:
                    description: Maximum prefix limit
                    type: int
                  map_import:
                    description: Route-map based VRF import
                    type: str
                  advertise_vpn:
                    description: Allow leaked routes to be advertised to VPN
                    type: bool
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the NX-OS device by
        executing the command B(show running-config | section ^vrf).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    choices: [parsed, gathered, deleted, purged, merged, replaced, rendered, overridden]
    default: merged
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of command I(show running-config | section ^vrf).
        connection to remote host is not required.
    type: str
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# nxos#show running-config | section ^vrf

- name: Merge provided configuration with device configuration
  register: result
  cisco.nxos.nxos_vrf_address_family:
    config:
      - name: VRF1
        address_families:
          - afi: ipv4
            safi: unicast
            route_target:
              - export: "65512:200"
            maximum:
              max_routes: 500
              max_route_options:
                threshold:
                  threshold_value: 60
                  reinstall_threshold: 80
            export:
              - map: "22"
              - vrf:
                  allow_vpn: true
                  map_import: "44"
              - vrf:
                  allow_vpn: true
          - afi: ipv6
            safi: unicast
            maximum:
              max_routes: 1000
            route_target:
              - import: "65512:200"
            import:
              - map: "22"
              - vrf:
                  advertise_vpn: true
                  map_import: "44"
              - vrf:
                  advertise_vpn: true
    state: merged

# Task Output:
# ------------

# before: {}
# commands:
#   - vrf context VRF1
#   - address-family ipv4 unicast
#   - maximum routes 500 60 reinstall 80
#   - route-target export 65512:200
#   - export map 22
#   - export vrf default map 44 allow-vpn
#   - export vrf allow-vpn
#   - address-family ipv6 unicast
#   - maximum routes 1000
#   - route-target import 65512:200
#   - import map 22
#   - import vrf default map 44 advertise-vpn
#   - import vrf advertise-vpn
# after:
#   - address_families:
#       - afi: ipv4
#         export:
#           - map: "22"
#           - vrf:
#               allow_vpn: true
#               map_import: "44"
#           - vrf:
#               allow_vpn: true
#         maximum:
#           max_route_options:
#             threshold:
#               reinstall_threshold: 80
#               threshold_value: 60
#           max_routes: 500
#         route_target:
#           - export: 65512:200
#         safi: unicast
#       - afi: ipv6
#         import:
#           - map: "22"
#           - vrf:
#               advertise_vpn: true
#               map_import: "44"
#           - vrf:
#               advertise_vpn: true
#         maximum:
#           max_routes: 1000
#         route_target:
#           - import: 65512:200
#         safi: unicast
#     name: VRF1

# After state:
# ------------
#
# nxos#show running-config | section ^vrf
# vrf context VRF1
#   address-family ipv4 unicast
#     route-target export 65512:200
#     export map 22
#     export vrf default map 44 allow-vpn
#     export vrf allow-vpn
#     maximum routes 500 60 reinstall 80
#   address-family ipv6 unicast
#     route-target import 65512:200
#     import map 22
#     import vrf default map 44 advertise-vpn
#     import vrf advertise-vpn
#     maximum routes 1000

# Using deleted

# Before state:
# -------------
#
# nxos#show running-config | section ^vrf
# vrf context VRF1
#   address-family ipv4 unicast
#      route-target import 64512:200
#      route-target export 64512:200
#      export map 22
#      export vrf default map 44 allow-vpn
#      export vrf allow-vpn
#      maximum routes 900 22 reinstall 44

- name: Delete given vrf address family configuration
  register: result
  cisco.nxos.nxos_vrf_address_family:
    config:
      - name: VRF1
        address_families:
          - afi: ipv4
            safi: unicast
            route_target:
              - import: 64512:200
            export:
              - map: "22"
            maximum:
              max_routes: 900
              max_route_options:
                threshold:
                  threshold_value: 22
                  reinstall_threshold: 44
    state: deleted

# Task Output:
# ------------
#
# before:
#  - address_families:
#      - afi: ipv4
#        export:
#          - map: "22"
#          - vrf:
#              allow_vpn: true
#              map_import: "44"
#          - vrf:
#              allow_vpn: true
#        maximum:
#          max_route_options:
#            threshold:
#              reinstall_threshold: 44
#              threshold_value: 22
#          max_routes: 900
#        route_target:
#          - import: "64512:200"
#          - export: "64512:200"
#        safi: unicast
#    name: VRF1

# commands:
#   - vrf context VRF1
#   - address-family ipv4 unicast
#   - no maximum routes 900 22 reinstall 44
#   - no route-target import 64512:200
#   - no export map 22
# after:
#   - address_families:
#       - afi: ipv4
#         export:
#           - vrf:
#               allow_vpn: true
#               map_import: "44"
#           - vrf:
#               allow_vpn: true
#         route_target:
#           - export: "64512:200"
#         safi: unicast
#     name: VRF1

# Using purged

# Before state:
# -------------
#
# nxos#show running-config | section ^vrf
# vrf context VRF1
#   address-family ipv4 unicast
#     route-target export 65512:200
#     export map 22
#     export vrf default map 44 allow-vpn
#     export vrf allow-vpn
#     maximum routes 500 60 reinstall 80
#   address-family ipv6 unicast
#     route-target import 65512:200
#     import map 22
#     import vrf default map 44 advertise-vpn
#     import vrf advertise-vpn
#     maximum routes 1000

- name: Purge the configuration of VRF address family
  register: result
  cisco.nxos.nxos_vrf_address_family:
    config:
      - name: VRF1
        address_families:
          - afi: ipv4
            safi: unicast
          - afi: ipv6
            safi: unicast
    state: purged

# Task Output:
# ------------
#
# before:
#     - address_families:
#           - afi: ipv4
#             export:
#                 - map: "22"
#                 - vrf:
#                       allow_vpn: true
#                       map_import: "44"
#                 - vrf:
#                       allow_vpn: true
#             maximum:
#                 max_route_options:
#                     threshold:
#                         reinstall_threshold: 80
#                         threshold_value: 60
#                 max_routes: 500
#             route_target:
#                 - export: 65512:200
#             safi: unicast
#           - afi: ipv6
#             import:
#                 - map: "22"
#                 - vrf:
#                       advertise_vpn: true
#                       map_import: "44"
#                 - vrf:
#                       advertise_vpn: true
#             maximum:
#                 max_routes: 1000
#             route_target:
#                 - import: 65512:200
#             safi: unicast
#       name: VRF1
# commands:
#   - vrf context VRF1
#   - no address-family ipv4 unicast
#   - no address-family ipv6 unicast
# after: {}


# Using overridden

# Before state:
# -------------
#
# nxos#show running-config | section ^vrf
# vrf context VRF1
#   address-family ipv4 unicast
#     route-target import 64512:200
#   address-family ipv6 unicast
#     route-target import 554832:500

- name: Override the provided configuration with the existing running configuration
  cisco.nxos.nxos_vrf_address_family:
    config:
      - name: VRF1
        address_families:
          - afi: ipv6
            safi: unicast
            route_target:
              - export: 65512:200
            maximum:
              max_routes: 500
              max_route_options:
                threshold:
                  threshold_value: 60
                  reinstall_threshold: 80
            export:
              - map: "22"
              - vrf:
                  allow_vpn: true
                  map_import: "44"
              - vrf:
                  allow_vpn: true
      - name: temp
        address_families:
          - afi: ipv4
            safi: unicast
            route_target:
              - import: 65512:200
            maximum:
              max_routes: 1000
            export:
              - map: "26"
              - vrf:
                  allow_vpn: true
                  map_import: "46"
    state: overridden

# Task Output:
# ------------
#
# before:
#  - address_families:
#      - afi: ipv4
#        route_target:
#          - import: 64512:200
#        safi: unicast
#      - afi: ipv6
#        route_target:
#          - import: 554832:500
#        safi: unicast
#    name: VRF1
#
# commands:
#  - vrf context VRF1
#  - address-family ipv4 unicast
#  - no route-target import 64512:200
#  - address-family ipv6 unicast
#  - maximum routes 500 60 reinstall 80
#  - no route-target import 554832:500
#  - route-target export 65512:200
#  - export map 22
#  - export vrf default map 44 allow-vpn
#  - export vrf allow-vpn
#  - vrf context temp
#  - address-family ipv4 unicast
#  - maximum routes 1000
#  - route-target import 65512:200
#  - export map 26
#  - export vrf default map 46 allow-vpn
# after:
#  - address_families:
#      - afi: ipv4
#        safi: unicast
#      - afi: ipv6
#        export:
#          - map: "22"
#          - vrf:
#              allow_vpn: true
#              map_import: "44"
#          - vrf:
#              allow_vpn: true
#        maximum:
#          max_route_options:
#            threshold:
#              reinstall_threshold: 80
#              threshold_value: 60
#          max_routes: 500
#        route_target:
#          - export: 65512:200
#        safi: unicast
#    name: VRF1
#  - address_families:
#      - afi: ipv4
#        export:
#          - map: "26"
#          - vrf:
#              allow_vpn: true
#              map_import: "46"
#        maximum:
#          max_routes: 1000
#        route_target:
#          - import: 65512:200
#        safi: unicast
#    name: temp

# Using replaced

# Before state:
# -------------
#
# nxos# show running-config | section ^vrf
# vrf context VRF1
#   address-family ipv4 unicast
#     route-target import 64512:200
#   address-family ipv6 unicast
#     route-target import 554832:500

- name: Replaced state for VRF configuration
  cisco.nxos.nxos_vrf_global:
    config:
      vrfs:
        - ip:
            name_server:
              address_list:
                - 192.168.255.1
            route:
              - destination: 192.168.255.1
                source: 0.0.0.0/0
          name: management
        - name: temp
          description: Test
          ip:
            auto_discard: true
            domain_list:
              - invalid.com
              - example.com
            domain_name: test.org
    state: replaced

# Task Output:
# ------------
#
# before:
#  - address_families:
#      - afi: ipv4
#        route_target:
#          - import: 64512:200
#        safi: unicast
#      - afi: ipv6
#        route_target:
#          - import: 554832:500
#        safi: unicast
#    name: VRF1
# commands:
#  - vrf context VRF1
#  - address-family ipv4 unicast
#  - no route-target import 64512:200
#  - address-family ipv6 unicast
#  - maximum routes 500 60 reinstall 80
#  - no route-target import 554832:500
#  - route-target export 65512:200
#  - export map 22
#  - export vrf default map 44 allow-vpn
#  - export vrf allow-vpn
#  - vrf context temp
#  - address-family ipv4 unicast
#  - maximum routes 1000
#  - route-target import 65512:200
#  - export map 26
#  - export vrf default map 46 allow-vpn
# after:
#  - address_families:
#      - afi: ipv4
#        safi: unicast
#      - afi: ipv6
#        export:
#          - map: "22"
#          - vrf:
#              allow_vpn: true
#              map_import: "44"
#          - vrf:
#              allow_vpn: true
#        maximum:
#          max_route_options:
#            threshold:
#              reinstall_threshold: 80
#              threshold_value: 60
#          max_routes: 500
#        route_target:
#          - export: 65512:200
#        safi: unicast
#    name: VRF1
#  - address_families:
#      - afi: ipv4
#        export:
#          - map: "26"
#          - vrf:
#              allow_vpn: true
#              map_import: "46"
#        maximum:
#          max_routes: 1000
#        route_target:
#          - import: 65512:200
#        safi: unicast
#    name: temp
#
# After state:
# ------------
# router-ios#show running-config | section ^vrf
# vrf context VRF1
#   address-family ipv6 unicast
#     route-target export 65512:200
#     export map 22
#     export vrf default map 44 allow-vpn
#     export vrf allow-vpn
# vrf context temp
#   address-family ipv4 unicast
#     route-target import 65512:200
#     export map 26
#     export vrf default map 46 allow-vpn
#     maximum routes 1000

# Using gathered

# Before state:
# -------------
#
# nxos#show running-config | section ^vrf
# vrf context VRF1
#   address-family ipv4 unicast
#     route-target export 65512:200
#     export map 22
#     export vrf default map 44 allow-vpn
#     export vrf allow-vpn
#     maximum routes 500 60 reinstall 80
#   address-family ipv6 unicast
#     route-target import 65512:200
#     import map 22
#     import vrf default map 44 advertise-vpn
#     import vrf advertise-vpn
#     maximum routes 1000

- name: Gathered state for VRF configuration
  cisco.nxos.nxos_vrf_global:
    config:
    state: gathered

# Task Output:
# ------------
#
# gathered:
#     - address_families:
#           - afi: ipv4
#             export:
#                 - map: "22"
#                 - vrf:
#                       allow_vpn: true
#                       map_import: "44"
#                 - vrf:
#                       allow_vpn: true
#             maximum:
#                 max_route_options:
#                     threshold:
#                         reinstall_threshold: 80
#                         threshold_value: 60
#                 max_routes: 500
#             route_target:
#                 - export: 65512:200
#             safi: unicast
#           - afi: ipv6
#             import:
#                 - map: "22"
#                 - vrf:
#                       advertise_vpn: true
#                       map_import: "44"
#                 - vrf:
#                       advertise_vpn: true
#             maximum:
#                 max_routes: 1000
#             route_target:
#                 - import: 65512:200
#             safi: unicast
#       name: VRF1

# Using rendered

- name: Render provided configuration with device configuration
  register: result
  cisco.nxos.nxos_vrf_address_family:
    config:
      - name: VRF1
        address_families:
          - afi: ipv6
            safi: unicast
            route_target:
              - export: 65512:200
            maximum:
              max_routes: 500
              max_route_options:
                threshold:
                  threshold_value: 60
                  reinstall_threshold: 80
            export:
              - map: "22"
              - vrf:
                  allow_vpn: true
                  map_import: "44"
              - vrf:
                  allow_vpn: true
      - name: temp
        address_families:
          - afi: ipv4
            safi: unicast
            route_target:
              - import: 65512:200
            maximum:
              max_routes: 1000
            export:
              - map: "26"
              - vrf:
                  allow_vpn: true
                  map_import: "46"
    state: rendered

# Task Output:
# ------------
#
# commands:
#   - vrf context VRF1
#   - address-family ipv6 unicast
#   - maximum routes 500 60 reinstall 80
#   - route-target export 65512:200
#   - export map 22
#   - export vrf default map 44 allow-vpn
#   - export vrf allow-vpn
#   - vrf context temp
#   - address-family ipv4 unicast
#   - maximum routes 1000
#   - route-target import 65512:200
#   - export map 26
#   - export vrf default map 46 allow-vpn

# Using Parsed

# Parsed Config:
# -------------
# vrf context VRF1
#   address-family ipv4 unicast
#     route-target import 64512:200
#     route-target export 64512:200
#     export map 22
#     export vrf default map 44 allow-vpn
#     export vrf allow-vpn
#     maximum routes 900 22 reinstall 44
#   address-family ipv6 unicast
#     route-target import 554832:500

- name: Parse the commands for provided configuration
  register: result
  cisco.nxos.nxos_vrf_address_family:
    running_config: "{{ lookup('file', '_parsed.cfg') }}"
    state: parsed

# Task Output:
# ------------
# parsed:
#   - name: VRF1
#     address_families:
#       - afi: ipv4
#         safi: unicast
#         route_target:
#           - import: 64512:200
#           - export: 64512:200
#         export:
#           - map: "22"
#           - vrf:
#               allow_vpn: true
#               map_import: "44"
#           - vrf:
#               allow_vpn: true
#         maximum:
#           max_routes: 900
#           max_route_options:
#             threshold:
#               threshold_value: 22
#               reinstall_threshold: 44
#       - afi: ipv6
#         safi: unicast
#         route_target:
#           - import: 554832:500
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - vrf context management
    - address-family ipv4 unicast
    - maximum routes 500 60 reinstall 80
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - vrf context test1
    - address-family ipv6 unicast
    - route-target export 65512:200
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.vrf_address_family.vrf_address_family import (
    Vrf_address_familyArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.vrf_address_family.vrf_address_family import (
    Vrf_address_family,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Vrf_address_familyArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Vrf_address_family(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
    author: Bradley Thornton (@cidrblock)
    name: native
    short_description: Define configurable options for C(native) sub-plugin of M(ansible.utils.fact_diff) module
    description:
    - This plugin documentation provides the configurable options that can be passed
      to the I(ansible.utils.fact_diff) plugins when I(ansible.utils.native) is used as a value for
      I(name) option of the module.
    version_added: 1.0.0
"""

EXAMPLES = r"""
- name: Show the difference in json format
  ansible.utils.fact_diff:
    before: "{{ before }}"
    after: "{{ after }}"
    plugin:
      name: ansible.utils.native
"""

import re

from ansible.plugins.callback import CallbackBase

from ansible_collections.ansible.utils.plugins.plugin_utils.base.fact_diff import FactDiffBase


class FactDiff(FactDiffBase):
    def _check_valid_regexes(self):
        if self._skip_lines:
            self._debug("Checking regex in 'split_lines' for validity")
            for idx, regex in enumerate(self._skip_lines):
                try:
                    self._skip_lines[idx] = re.compile(regex)
                except re.error as exc:
                    msg = "The regex '{regex}', is not valid. The error was {err}.".format(
                        regex=regex,
                        err=str(exc),
                    )
                    self._errors.append(msg)

    def _xform(self):
        if self._skip_lines:
            if isinstance(self._before, str):
                self._debug("'before' is a string, splitting lines")
                self._before = self._before.splitlines()
            if isinstance(self._after, str):
                self._debug("'after' is a string, splitting lines")
                self._after = self._after.splitlines()
            self._before = [
                line
                for line in self._before
                if not any(regex.match(str(line)) for regex in self._skip_lines)
            ]
            self._after = [
                line
                for line in self._after
                if not any(regex.match(str(line)) for regex in self._skip_lines)
            ]
        if isinstance(self._before, list):
            self._debug("'before' is a list, joining with \n")
            self._before = "\n".join(map(str, self._before)) + "\n"
        if isinstance(self._after, list):
            self._debug("'after' is a list, joining with \n")
            self._after = "\n".join(map(str, self._after)) + "\n"

    def diff(self):
        self._after = self._task_args["after"]
        self._before = self._task_args["before"]
        self._errors = []
        self._skip_lines = self._task_args["plugin"]["vars"].get("skip_lines")
        self._check_valid_regexes()
        if self._errors:
            return {"errors": " ".join(self._errors)}
        self._xform()
        diff = CallbackBase()._get_diff({"before": self._before, "after": self._after})
        return {"diff": diff}



# Copyright (c) 2021, RevBits <info@revbits.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
name: revbitspss
author: RevBits (@RevBits) <info@revbits.com>
short_description: Get secrets from RevBits PAM server
version_added: 4.1.0
description:
  - Uses the revbits_ansible Python SDK to get Secrets from RevBits PAM Server using API key authentication with the REST
    API.
requirements:
  - revbits_ansible - U(https://pypi.org/project/revbits_ansible/)
options:
  _terms:
    description:
      - This is an array of keys for secrets which you want to fetch from RevBits PAM.
    required: true
    type: list
    elements: string
  base_url:
    description:
      - This is the base URL of the server, for example V(https://server-url-here).
    required: true
    type: string
  api_key:
    description:
      - This is the API key for authentication. You can get it from the RevBits PAM secret manager module.
    required: true
    type: string
"""

RETURN = r"""
_list:
  description:
    - The JSON responses which you can access with defined keys.
    - If you are fetching secrets named as UUID, PASSWORD it returns the dict of all secrets.
  type: list
  elements: dict
"""

EXAMPLES = r"""
---
- hosts: localhost
  vars:
    secret: >-
      {{
        lookup(
          'community.general.revbitspss',
          'UUIDPAM', 'DB_PASS',
          base_url='https://server-url-here',
          api_key='API_KEY_GOES_HERE'
        )
      }}
  tasks:
    - ansible.builtin.debug:
        msg: >-
          UUIDPAM is {{ (secret['UUIDPAM']) }} and DB_PASS is {{ (secret['DB_PASS']) }}
"""

from ansible.errors import AnsibleError
from ansible.plugins.lookup import LookupBase
from ansible.utils.display import Display

from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms

ANOTHER_LIBRARY_IMPORT_ERROR: ImportError | None
try:
    from pam.revbits_ansible.server import SecretServer
except ImportError as imp_exc:
    ANOTHER_LIBRARY_IMPORT_ERROR = imp_exc
else:
    ANOTHER_LIBRARY_IMPORT_ERROR = None


display = Display()


class LookupModule(LookupBase):
    @staticmethod
    def Client(server_parameters):
        return SecretServer(**server_parameters)

    def run(self, terms, variables, **kwargs):
        if ANOTHER_LIBRARY_IMPORT_ERROR:
            raise AnsibleError("revbits_ansible must be installed to use this plugin") from ANOTHER_LIBRARY_IMPORT_ERROR
        self.set_options(var_options=variables, direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)
        secret_server = LookupModule.Client(
            {
                "base_url": self.get_option("base_url"),
                "api_key": self.get_option("api_key"),
            }
        )
        result = []
        for term in terms:
            try:
                display.vvv(f"Secret Server lookup of Secret with ID {term}")
                result.append({term: secret_server.get_pam_secret(term)})
            except Exception as error:
                raise AnsibleError(f"Secret Server lookup failure: {error.message}") from error
        return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2022 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: eks_nodegroup
version_added: 5.3.0
short_description: Manage EKS Nodegroup module
description:
  - Manage EKS Nodegroup.
author:
  - Tiago Jarra (@tjarra)
options:
  name:
    description: Name of EKS Nodegroup.
    required: True
    type: str
  cluster_name:
    description: Name of EKS Cluster.
    required: True
    type: str
  node_role:
    description: ARN of IAM role used by the EKS cluster Nodegroup.
    type: str
  subnets:
    description: list of subnet IDs for the Kubernetes cluster.
    type: list
    elements: str
  scaling_config:
    description: The scaling configuration details for the Auto Scaling group that is created for your node group.
    type: dict
    default:
      min_size: 1
      max_size: 2
      desired_size: 1
    suboptions:
      min_size:
        description: The minimum number of nodes that the managed node group can scale in to.
        type: int
      max_size:
        description: The maximum number of nodes that the managed node group can scale out to.
        type: int
      desired_size:
        description: The current number of nodes that the managed node group should maintain.
        type: int
  disk_size:
    description:
      - Size of disk in nodegroup nodes.
        If you specify I(launch_template), then don't specify I(disk_size), or the node group deployment will fail.
    type: int
  instance_types:
    description:
      - Specify the instance types for a node group.
        If you specify I(launch_template), then don't specify I(instance_types), or the node group deployment will fail.
    type: list
    elements: str
  ami_type:
    description: The AMI type for your node group.
    type: str
    choices:
      - AL2_x86_64
      - AL2_x86_64_GPU
      - AL2_ARM_64
      - CUSTOM
      - BOTTLEROCKET_ARM_64
      - BOTTLEROCKET_x86_64
  remote_access:
    description:
      - The remote access (SSH) configuration to use with your node group.
        If you specify I(launch_template), then don't specify I(remote_access), or the node group deployment will fail.
    type: dict
    suboptions:
      ec2_ssh_key:
        description: The Amazon EC2 SSH key that provides access for SSH communication with the nodes in the managed node group.
        type: str
      source_sg:
        description: The security groups that are allowed SSH access (port 22) to the nodes.
        type: list
        elements: str
  update_config:
    description: The node group update configuration.
    type: dict
    default:
      max_unavailable: 1
    suboptions:
      max_unavailable:
        description: The maximum number of nodes unavailable at once during a version update.
        type: int
      max_unavailable_percentage:
        description: The maximum percentage of nodes unavailable during a version update.
        type: int
  labels:
    description: The Kubernetes labels to be applied to the nodes in the node group when they are created.
    type: dict
    default: {}
  taints:
    description: The Kubernetes taints to be applied to the nodes in the node group.
    type: list
    elements: dict
    default: []
    suboptions:
      key:
        description: The key of the taint.
        type: str
      value:
        description: The value of the taint.
        type: str
      effect:
        description: The effect of the taint.
        type: str
        choices:
          - NO_SCHEDULE
          - NO_EXECUTE
          - PREFER_NO_SCHEDULE
  launch_template:
    description:
      - An object representing a node group's launch template specification.
      - If specified, then do not specify I(instanceTypes), I(diskSize), or I(remoteAccess).
    type: dict
    suboptions:
      name:
        description: The name of the launch template.
        type: str
      version:
        description:
          - The version of the launch template to use.
          - If no version is specified, then the template's default version is used.
        type: str
      id:
        description: The ID of the launch template.
        type: str
  capacity_type:
    description: The capacity type for your node group.
    default: ON_DEMAND
    type: str
    choices:
      - ON_DEMAND
      - SPOT
  release_version:
    description: The AMI version of the Amazon EKS optimized AMI to use with your node group.
    type: str
  state:
    description: Create or delete the Nodegroup.
    choices:
      - absent
      - present
    default: present
    type: str
  tags:
    description: A dictionary of resource tags.
    type: dict
    aliases: ['resource_tags']
  purge_tags:
    description:
      - Purge existing tags that are not found in the nodegroup.
    type: bool
    default: true
  wait:
    description: Specifies whether the module waits until the profile is created or deleted before moving on.
    type: bool
    default: false
  wait_timeout:
    description: The duration in seconds to wait for the nodegroup to become active. Defaults to C(1200) seconds.
    default: 1200
    type: int
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: create nodegroup
  community.aws.eks_nodegroup:
    name: test_nodegroup
    state: present
    cluster_name: test_cluster
    node_role: arn:aws:eks:us-east-1:1231231123:role/asdf
    subnets:
      - subnet-qwerty123
      - subnet-asdfg456
    scaling_config:
      min_size: 1
      max_size: 2
      desired_size: 1
    disk_size: 20
    instance_types: 't3.micro'
    ami_type: 'AL2_x86_64'
    labels:
      'teste': 'test'
    taints:
      - key: 'test'
        value: 'test'
        effect: 'NO_SCHEDULE'
    capacity_type: 'ON_DEMAND'

- name: Remove an EKS Nodegrop
  community.aws.eks_nodegroup:
    name: test_nodegroup
    cluster_name: test_cluster
    wait: true
    state: absent
"""

RETURN = r"""
nodegroup_name:
  description: The name associated with an Amazon EKS managed node group.
  returned: when state is present
  type: str
  sample: test_cluster
nodegroup_arn:
  description: The Amazon Resource Name (ARN) associated with the managed node group.
  returned: when state is present
  type: str
  sample: arn:aws:eks:us-east-1:1231231123:safd
cluster_name:
  description: Name of EKS Cluster
  returned: when state is present
  type: str
  sample: test_cluster
version:
  description: The Kubernetes version of the managed node group.
  returned: when state is present
  type: str
  sample: need_validate
release_version:
  description: This is the version of the Amazon EKS optimized AMI that the node group was deployed with.
  returned: when state is present
  type: str
  sample: need_validate
created_at:
  description: Nodegroup creation date and time.
  returned: when state is present
  type: str
  sample: '2022-01-18T20:00:00.111000+00:00'
modified_at:
  description: Nodegroup modified date and time.
  returned: when state is present
  type: str
  sample: '2022-01-18T20:00:00.111000+00:00'
status:
  description: status of the EKS Nodegroup.
  returned: when state is present
  type: str
  sample:
  - CREATING
  - ACTIVE
capacity_type:
  description: The capacity type of your managed node group.
  returned: when state is present
  type: str
  sample: need_validate
scaling_config:
  description: The scaling configuration details for the Auto Scaling group that is associated with your node group.
  returned: when state is present
  type: dict
  sample: need_validate
instance_types:
  description: This is the instance type that is associated with the node group.
  returned: when state is present
  type: list
  sample: need_validate
subnets:
  description: List of subnets used in Fargate Profile.
  returned: when state is present
  type: list
  sample:
  - subnet-qwerty123
  - subnet-asdfg456
remote_access:
  description: This is the remote access configuration that is associated with the node group.
  returned: when state is present
  type: dict
  sample: need_validate
ami_type:
  description: This is the AMI type that was specified in the node group configuration.
  returned: when state is present
  type: str
  sample: need_validate
node_role:
  description: ARN of the IAM Role used by Nodegroup.
  returned: when state is present
  type: str
  sample: arn:aws:eks:us-east-1:1231231123:role/asdf
labels:
  description: The Kubernetes labels applied to the nodes in the node group.
  returned: when state is present
  type: dict
  sample: need_validate
taints:
  description: The Kubernetes taints to be applied to the nodes in the node group when they are created.
  returned: when state is present
  type: list
  sample: need_validate
resources:
  description: The resources associated with the node group.
  returned: when state is present
  type: complex
  contains:
    autoScalingGroups:
      description: The Auto Scaling groups associated with the node group.
      returned: when state is present
      type: list
      elements: dict
    remoteAccessSecurityGroup:
      description: The remote access security group associated with the node group.
      returned: when state is present
      type: str
diskSize:
  description: This is the disk size in the node group configuration.
  returned:  when state is present
  type: int
  sample: 20
health:
  description: The health status of the node group.
  returned: when state is present
  type: dict
  sample: need_validate
update_config:
  description: The node group update configuration.
  returned: when state is present
  type: dict
  contains:
    maxUnavailable:
      description: The maximum number of nodes unavailable at once during a version update.
      type: int
    maxUnavailablePercentage:
      description: The maximum percentage of nodes unavailable during a version update.
      type: int
launch_template:
  description: If a launch template was used to create the node group, then this is the launch template that was used.
  returned: when state is present
  type: dict
  sample: need_validate
tags:
  description: Nodegroup tags.
  returned: when state is present
  type: dict
  sample:
    foo: bar
"""

try:
    import botocore
except ImportError:
    pass

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags
from ansible_collections.amazon.aws.plugins.module_utils.waiters import get_waiter

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def validate_tags(client, module, nodegroup):
    changed = False

    desired_tags = module.params.get("tags")
    if desired_tags is None:
        return False

    try:
        existing_tags = client.list_tags_for_resource(resourceArn=nodegroup["nodegroupArn"])["tags"]
        tags_to_add, tags_to_remove = compare_aws_tags(existing_tags, desired_tags, module.params.get("purge_tags"))
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg=f"Unable to list or compare tags for Nodegroup {module.params.get('name')}.")
    if tags_to_remove:
        if not module.check_mode:
            changed = True
            try:
                client.untag_resource(aws_retry=True, ResourceArn=nodegroup["nodegroupArn"], tagKeys=tags_to_remove)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg=f"Unable to set tags for Nodegroup {module.params.get('name')}.")
    if tags_to_add:
        if not module.check_mode:
            changed = True
            try:
                client.tag_resource(aws_retry=True, ResourceArn=nodegroup["nodegroupArn"], tags=tags_to_add)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg=f"Unable to set tags for Nodegroup {module.params.get('name')}.")

    return changed


def compare_taints(nodegroup_taints, param_taints):
    taints_to_unset = []
    taints_to_add_or_update = []
    for taint in nodegroup_taints:
        if taint not in param_taints:
            taints_to_unset.append(taint)
    for taint in param_taints:
        if taint not in nodegroup_taints:
            taints_to_add_or_update.append(taint)

    return taints_to_add_or_update, taints_to_unset


def validate_taints(client, module, nodegroup, param_taints):
    changed = False
    params = dict()
    params["clusterName"] = nodegroup["clusterName"]
    params["nodegroupName"] = nodegroup["nodegroupName"]
    params["taints"] = []
    if "taints" not in nodegroup:
        nodegroup["taints"] = []
    taints_to_add_or_update, taints_to_unset = compare_taints(nodegroup["taints"], param_taints)

    if taints_to_add_or_update:
        params["taints"]["addOrUpdateTaints"] = taints_to_add_or_update
    if taints_to_unset:
        params["taints"]["removeTaints"] = taints_to_unset
    if params["taints"]:
        if not module.check_mode:
            changed = True
            try:
                client.update_nodegroup_config(**params)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg=f"Unable to set taints for Nodegroup {params['nodegroupName']}.")

    return changed


def compare_labels(nodegroup_labels, param_labels):
    labels_to_unset = []
    labels_to_add_or_update = {}
    for label in nodegroup_labels.keys():
        if label not in param_labels:
            labels_to_unset.append(label)
    for key, value in param_labels.items():
        if key not in nodegroup_labels.keys():
            labels_to_add_or_update[key] = value

    return labels_to_add_or_update, labels_to_unset


def validate_labels(client, module, nodegroup, param_labels):
    changed = False
    params = dict()
    params["clusterName"] = nodegroup["clusterName"]
    params["nodegroupName"] = nodegroup["nodegroupName"]
    params["labels"] = {}
    labels_to_add_or_update, labels_to_unset = compare_labels(nodegroup["labels"], param_labels)

    if labels_to_add_or_update:
        params["labels"]["addOrUpdateLabels"] = labels_to_add_or_update
    if labels_to_unset:
        params["labels"]["removeLabels"] = labels_to_unset
    if params["labels"]:
        if not module.check_mode:
            changed = True
            try:
                client.update_nodegroup_config(**params)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg=f"Unable to set labels for Nodegroup {params['nodegroupName']}.")

    return changed


def compare_params(module, params, nodegroup):
    for param in ["nodeRole", "subnets", "diskSize", "instanceTypes", "amiTypes", "remoteAccess", "capacityType"]:
        if (param in nodegroup) and (param in params):
            if nodegroup[param] != params[param]:
                module.fail_json(msg=f"Cannot modify parameter {param}.")
    if ("launchTemplate" not in nodegroup) and ("launchTemplate" in params):
        module.fail_json(msg="Cannot add Launch Template in this Nodegroup.")
    if nodegroup["updateConfig"] != params["updateConfig"]:
        return True
    if nodegroup["scalingConfig"] != params["scalingConfig"]:
        return True
    return False


def compare_params_launch_template(module, params, nodegroup):
    if "launchTemplate" not in params:
        module.fail_json(msg="Cannot exclude Launch Template in this Nodegroup.")
    else:
        for key in ["name", "id"]:
            if (key in params["launchTemplate"]) and (
                params["launchTemplate"][key] != nodegroup["launchTemplate"][key]
            ):
                module.fail_json(msg=f"Cannot modify Launch Template {key}.")
        if ("version" in params["launchTemplate"]) and (
            params["launchTemplate"]["version"] != nodegroup["launchTemplate"]["version"]
        ):
            return True
    return False


def create_or_update_nodegroups(client, module):
    changed = False
    params = dict()
    params["nodegroupName"] = module.params["name"]
    params["clusterName"] = module.params["cluster_name"]
    params["nodeRole"] = module.params["node_role"]
    params["subnets"] = module.params["subnets"]
    params["tags"] = module.params["tags"] or {}
    if module.params["ami_type"] is not None:
        params["amiType"] = module.params["ami_type"]
    if module.params["disk_size"] is not None:
        params["diskSize"] = module.params["disk_size"]
    if module.params["instance_types"] is not None:
        params["instanceTypes"] = module.params["instance_types"]
    if module.params["launch_template"] is not None:
        params["launchTemplate"] = dict()
        if module.params["launch_template"]["id"] is not None:
            params["launchTemplate"]["id"] = module.params["launch_template"]["id"]
        if module.params["launch_template"]["version"] is not None:
            params["launchTemplate"]["version"] = module.params["launch_template"]["version"]
        if module.params["launch_template"]["name"] is not None:
            params["launchTemplate"]["name"] = module.params["launch_template"]["name"]
    if module.params["release_version"] is not None:
        params["releaseVersion"] = module.params["release_version"]
    if module.params["remote_access"] is not None:
        params["remoteAccess"] = dict()
        if module.params["remote_access"]["ec2_ssh_key"] is not None:
            params["remoteAccess"]["ec2SshKey"] = module.params["remote_access"]["ec2_ssh_key"]
        if module.params["remote_access"]["source_sg"] is not None:
            params["remoteAccess"]["sourceSecurityGroups"] = module.params["remote_access"]["source_sg"]
    if module.params["capacity_type"] is not None:
        params["capacityType"] = module.params["capacity_type"].upper()
    if module.params["labels"] is not None:
        params["labels"] = module.params["labels"]
    if module.params["taints"] is not None:
        params["taints"] = module.params["taints"]
    if module.params["update_config"] is not None:
        params["updateConfig"] = dict()
        if module.params["update_config"]["max_unavailable"] is not None:
            params["updateConfig"]["maxUnavailable"] = module.params["update_config"]["max_unavailable"]
        if module.params["update_config"]["max_unavailable_percentage"] is not None:
            params["updateConfig"]["maxUnavailablePercentage"] = module.params["update_config"][
                "max_unavailable_percentage"
            ]
    if module.params["scaling_config"] is not None:
        params["scalingConfig"] = snake_dict_to_camel_dict(module.params["scaling_config"])

    wait = module.params.get("wait")
    nodegroup = get_nodegroup(client, module, params["nodegroupName"], params["clusterName"])

    if nodegroup:
        update_params = dict()
        update_params["clusterName"] = params["clusterName"]
        update_params["nodegroupName"] = params["nodegroupName"]

        if "launchTemplate" in nodegroup:
            if compare_params_launch_template(module, params, nodegroup):
                update_params["launchTemplate"] = params["launchTemplate"]
                if not module.check_mode:
                    try:
                        client.update_nodegroup_version(**update_params)
                    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
                        module.fail_json_aws(e, msg="Couldn't update nodegroup.")
                changed |= True

        if compare_params(module, params, nodegroup):
            try:
                if "launchTemplate" in update_params:
                    update_params.pop("launchTemplate")
                update_params["scalingConfig"] = params["scalingConfig"]
                update_params["updateConfig"] = params["updateConfig"]

                if not module.check_mode:
                    client.update_nodegroup_config(**update_params)

                changed |= True

            except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
                module.fail_json_aws(e, msg="Couldn't update nodegroup.")

        changed |= validate_tags(client, module, nodegroup)

        changed |= validate_labels(client, module, nodegroup, params["labels"])

        if "taints" in nodegroup:
            changed |= validate_taints(client, module, nodegroup, params["taints"])

        if wait:
            wait_until(client, module, "nodegroup_active", params["nodegroupName"], params["clusterName"])

        nodegroup = get_nodegroup(client, module, params["nodegroupName"], params["clusterName"])

        module.exit_json(changed=changed, **camel_dict_to_snake_dict(nodegroup))

    if module.check_mode:
        module.exit_json(changed=True)

    try:
        nodegroup = client.create_nodegroup(**params)
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, msg=f"Couldn't create Nodegroup {params['nodegroupName']}.")

    if wait:
        wait_until(client, module, "nodegroup_active", params["nodegroupName"], params["clusterName"])
        nodegroup = get_nodegroup(client, module, params["nodegroupName"], params["clusterName"])

    module.exit_json(changed=True, **camel_dict_to_snake_dict(nodegroup))


def delete_nodegroups(client, module):
    name = module.params.get("name")
    clusterName = module.params["cluster_name"]
    existing = get_nodegroup(client, module, name, clusterName)
    wait = module.params.get("wait")

    if not existing:
        module.exit_json(changed=False, msg="Nodegroup '{name}' does not exist")

    if existing["status"] == "DELETING":
        if wait:
            wait_until(client, module, "nodegroup_deleted", name, clusterName)
            module.exit_json(changed=False, msg="Nodegroup '{name}' deletion complete")
        module.exit_json(changed=False, msg="Nodegroup '{name}' already in DELETING state")

    if module.check_mode:
        module.exit_json(changed=True, msg="Nodegroup '{name}' deletion would be started (check mode)")

    try:
        client.delete_nodegroup(clusterName=clusterName, nodegroupName=name)
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, msg=f"Couldn't delete Nodegroup '{name}'.")

    if wait:
        wait_until(client, module, "nodegroup_deleted", name, clusterName)
        module.exit_json(changed=True, msg="Nodegroup '{name}' deletion complete")

    module.exit_json(changed=True, msg="Nodegroup '{name}' deletion started")


def get_nodegroup(client, module, nodegroup_name, cluster_name):
    try:
        return client.describe_nodegroup(clusterName=cluster_name, nodegroupName=nodegroup_name)["nodegroup"]
    except is_boto3_error_code("ResourceNotFoundException"):
        return None
    except (
        botocore.exceptions.BotoCoreError,
        botocore.exceptions.ClientError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg=f"Couldn't get Nodegroup {nodegroup_name}.")


def wait_until(client, module, waiter_name, nodegroup_name, cluster_name):
    wait_timeout = module.params.get("wait_timeout")
    waiter = get_waiter(client, waiter_name)
    attempts = 1 + int(wait_timeout / waiter.config.delay)
    try:
        waiter.wait(clusterName=cluster_name, nodegroupName=nodegroup_name, WaiterConfig={"MaxAttempts": attempts})
    except botocore.exceptions.WaiterError as e:
        module.fail_json_aws(e, msg="An error occurred waiting")


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        cluster_name=dict(type="str", required=True),
        node_role=dict(),
        subnets=dict(type="list", elements="str"),
        scaling_config=dict(
            type="dict",
            default={"min_size": 1, "max_size": 2, "desired_size": 1},
            options=dict(
                min_size=dict(type="int"),
                max_size=dict(type="int"),
                desired_size=dict(type="int"),
            ),
        ),
        disk_size=dict(type="int"),
        instance_types=dict(type="list", elements="str"),
        ami_type=dict(
            choices=[
                "AL2_x86_64",
                "AL2_x86_64_GPU",
                "AL2_ARM_64",
                "CUSTOM",
                "BOTTLEROCKET_ARM_64",
                "BOTTLEROCKET_x86_64",
            ]
        ),
        remote_access=dict(
            type="dict",
            options=dict(
                ec2_ssh_key=dict(no_log=True),
                source_sg=dict(type="list", elements="str"),
            ),
        ),
        update_config=dict(
            type="dict",
            default={"max_unavailable": 1},
            options=dict(
                max_unavailable=dict(type="int"),
                max_unavailable_percentage=dict(type="int"),
            ),
        ),
        labels=dict(type="dict", default={}),
        taints=dict(
            type="list",
            elements="dict",
            default=[],
            options=dict(
                key=dict(
                    type="str",
                    no_log=False,
                ),
                value=dict(type="str"),
                effect=dict(type="str", choices=["NO_SCHEDULE", "NO_EXECUTE", "PREFER_NO_SCHEDULE"]),
            ),
        ),
        launch_template=dict(
            type="dict",
            options=dict(
                name=dict(type="str"),
                version=dict(type="str"),
                id=dict(type="str"),
            ),
        ),
        capacity_type=dict(choices=["ON_DEMAND", "SPOT"], default="ON_DEMAND"),
        release_version=dict(),
        tags=dict(type="dict", aliases=["resource_tags"]),
        purge_tags=dict(type="bool", default=True),
        state=dict(choices=["absent", "present"], default="present"),
        wait=dict(default=False, type="bool"),
        wait_timeout=dict(default=1200, type="int"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_if=[["state", "present", ["node_role", "subnets"]]],
        mutually_exclusive=[
            ("launch_template", "instance_types"),
            ("launch_template", "disk_size"),
            ("launch_template", "remote_access"),
            ("launch_template", "ami_type"),
        ],
        supports_check_mode=True,
    )

    if module.params["launch_template"] is None:
        if module.params["disk_size"] is None:
            module.params["disk_size"] = 20
        if module.params["ami_type"] is None:
            module.params["ami_type"] = "AL2_x86_64"
        if module.params["instance_types"] is None:
            module.params["instance_types"] = ["t3.medium"]
    else:
        if (module.params["launch_template"]["id"] is None) and (module.params["launch_template"]["name"] is None):
            module.exit_json(changed=False, msg="To use launch_template, it is necessary to inform the id or name.")
    try:
        client = module.client("eks")
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, msg="Couldn't connect to AWS.")

    if module.params.get("state") == "present":
        create_or_update_nodegroups(client, module)
    else:
        delete_nodegroups(client, module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_dedicatedhost_info

version_added: "3.8.0"

short_description: Retrieves information abount dedicated host

description:
    - Retrieves information about a dedicated host.
    - Lists all of the dedicated hosts in the specified dedicated host group.

options:
    resource_group:
        description:
            - Name of the resource group.
        type: str
        required: True
    host_group_name:
        description:
            - The name of the dedicated host group.
        type: str
        required: True
    name:
        description:
            - Name of the dedicate host.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - magodo (@magodo)
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: List facts for one dedicate host
  azure_rm_dedicatedhost_info:
    resource_group: myAzureResourceGroup
    host_group_name: myhostgroup

- name: Get facts for dedicate host
  azure_rm_dedicatedhost_info:
    resource_group: myAzureResourceGroup
    host_group_name: myhostgroup
    name: mydedicatehost
'''

RETURN = '''
dedicate_hosts:
    description:
        - Gets a list of dedicated host.
    returned: always
    type: complex
    contains:
        host_group_name:
            description:
                - The name of the host group.
            type: str
            returned: always
            sample: myDedicatedHostGroup
        resource_group:
            description:
                - The name of the resource group.
            type: str
            returned: always
            sample: myResourceGroup
        name:
            description:
                - The dedicate hsot name.
            type: str
            returned: always
            sample: myHost
        id:
            description:
                - The dedicate host ID.
            type: str
            returned: always
            sample: "/subscriptions/{subscription-id}/resourceGroups/myResourceGroup/providers/Microsoft.Compute/HostGroups/myDedicatedHostGroup/hosts/myHost"
        location:
            description:
                - Resource location.
            type: str
            returned: always
            sample: eastus
        tags:
            description:
                - Resource tags.
            type: dict
            returned: always
            sample: {'key1': 'value1'}
        sku:
            description:
                - SKU of the dedicated host for Hardware Generation and VM family.
            type: dict
            returned: always
            sample: {'name': 'DSv3-Type4'}
        auto_replace_on_failure:
            description:
                - Specifies whether the dedicated host should be replaced automatically in case of a failure.
            type: bool
            returned: always
            sample: true
        license_type:
            description:
                - Specifies the software license type that will be applied to the VMs deployed on the dedicated host.
            type: str
            returned: always
            sample: None
        provisioning_state:
            description:
                - The provisioning state, which only appears in the response.
            type: str
            returned: always
            sample: Succeeded
        platform_fault_domain:
            description:
                - Fault domain of the dedicated host within a dedicated host group.
            type: str
            returned: always
            sample: 1
        virtual_machines:
            description:
                - A list of references to all virtual machines in the Dedicated Host.
            type: str
            returned: always
            sample: [{"id": "/subscriptions/subId/resourceGroups/myResourceGroup/providers/Microsoft.Compute/virtualMachines/vm1"}]
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

AZURE_OBJECT_CLASS = 'DedicateHostInfo'


class AzureRMDedicateHostInfo(AzureRMModuleBase):

    def __init__(self):

        # define user inputs into argument
        self.module_arg_spec = dict(
            name=dict(type='str'),
            host_group_name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True),
            tags=dict(type='list', elements='str')
        )

        # store the results of the module operation
        self.results = dict(
            changed=False
        )

        self.name = None
        self.resource_group = None
        self.host_group_name = None
        self.tags = None

        super(AzureRMDedicateHostInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=True)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        results = []
        if self.name is not None:
            # if there is a host name provided, return facts about that dedicated host
            results = self.get_item()
        else:
            # all the host listed in specific host group
            results = self.list_by_host_group()

        self.results['dedicated_hosts'] = [self.host_to_dict(item) for item in results] if results else []

        return self.results

    def get_item(self):
        self.log('Get host facts for {0}'.format(self.name))
        results = []
        # get specific host group
        try:
            item = self.compute_client.dedicated_hosts.get(self.resource_group, self.host_group_name, self.name)
        except ResourceNotFoundError:
            return []

        # serialize result
        if item and self.has_tags(item.tags, self.tags):
            results = [item]
        return results

    def list_by_host_group(self):
        self.log('List all host for host group - {0}'.format(self.host_group_name))
        try:
            response = self.compute_client.dedicated_hosts.list_by_host_group(self.resource_group, self.host_group_name)
        except Exception:
            return []

        results = []
        for item in response:
            if self.has_tags(item.tags, self.tags):
                results.append(item)
        return results

    def host_to_dict(self, host):
        result = dict(
            resource_group=self.resource_group,
            host_group_name=self.host_group_name,
            id=host.id,
            name=host.name,
            location=host.location,
            tags=host.tags,
            sku=dict(),
            auto_replace_on_failure=host.auto_replace_on_failure,
            license_type=host.license_type,
            provisioning_state=host.provisioning_state,
            platform_fault_domain=host.platform_fault_domain,
            virtual_machines=[]
        )
        if host.virtual_machines:
            result['virtual_machines'] = [dict(id=item.id) for item in host.virtual_machines]

        if host.sku:
            result['sku']['name'] = host.sku.name
        return result


def main():
    AzureRMDedicateHostInfo()


if __name__ == '__main__':
    main()
